/*
 * Copyright (c) 1993-2008 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/**
 ** platineCmdTaskCodels.c
 **
 ** Codels appeles par la tache platineCmdTask
 **
 ** Auteur: 
 ** Date: 
 **
 **/

#include <math.h>
#include <stdio.h>
#include "h2mathLib.h"

#include <portLib.h>

/* #include <timers.h> */

#include "server/platineHeader.h"

#include "driver/ptClient.h"

#include "platineCodels.h"
#include "platineCartesian.h"

#include "libeuler/pomEuler.h"

/*  struct timespec tp;
    u_long msec1,msec2;
    double sec1,sec2; 
    
    clock_gettime(CLOCK_REALTIME,&tp);
    msec1 = ((tp.tv_nsec / 1000000) + (tp.tv_sec * 1000));
    sec1 = ((double)(msec1))/1000; */

/* Calcul du temps de calcul */
/*	  clock_gettime(CLOCK_REALTIME,&tp);
	  msec2 = (tp.tv_nsec / 1000000) + (tp.tv_sec * 1000);
	  sec2 = ((double)(msec2))/1000;
	  printf ("TEMPS: %f s\n",sec2-sec1);
	  sec1 = sec2;
*/

static BOOL testStopEnd(PLATINE_AXIS axis, int pos);
static BOOL testSpeed(PLATINE_AXIS axis, int speed);
static BOOL testSpeedLimits(PLATINE_AXIS axis, int speed);
static BOOL testInit(int *bilan);

static PLATINE_REF_COORD_POS_STR platineCoordRef;
static POSTER_ID platineOriginFramePosterId;

#ifdef OLDVERSION
static PT_CMD_STATUS platineStopStatusP, platineStopStatusT;
#endif

/*------------------------------------------------------------------------
 * SetSpeed
 *
 * Description: 
 *
 * Bilans:      OK 
 */

/* platinetSetSpeed  -  codel EXEC de SetSpeed
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT platineSetSpeed (PLATINE_SET_PARAM *param, int *bilan)
{
  double speed;

  /* Test driver init */
  if (!testInit(bilan))
    return ETHER;

  /* Unity */
  if (param->unity == PLATINE_RAD)
    speed = param->val;
  else
    speed = DEG_TO_RAD(param->val);

  /* Test */
  if (!testSpeed(param->axis, PT_RAD_TO_STEP(speed,ptDriverSDI))) {
    *bilan = S_platine_INVALID_PARAMS;
    return ETHER;
  }

  /* Envoi la commande */
  if (param->axis == PLATINE_PAN) {
    SDI_F->platineParams.speedProfile.pan.speed = speed;
    ptSendCmdRad(SET_SPEED_PAN, ((double)SDI_F->driverParams.panTrigoRotation) * speed, ptDriverSDI);
  }
  else {
    SDI_F->platineParams.speedProfile.tilt.speed = speed;
    ptSendCmdRad(SET_SPEED_TILT, speed, ptDriverSDI);
  }

  return END;
}

ACTIVITY_EVENT platineSetSpeedEnd (PLATINE_SET_PARAM *param, int *bilan)
{
  PT_CMD_STATUS status;
  PT_ERR err;

  if (param->axis == PLATINE_PAN)
    status = ptGetStatus(SET_SPEED_PAN, &err, ptDriverSDI);
  else 
    status = ptGetStatus(SET_SPEED_TILT, &err, ptDriverSDI);

  if (status == PT_WAITING_REPLY)
    return END;

  if (status != PT_REPLY_OK) 
    *bilan = platineConvertPtErr(err);
  return ETHER;
}
ACTIVITY_EVENT platineSetSpeedInter (PLATINE_SET_PARAM *param, int *bilan)
{
  if (param->axis == PLATINE_PAN) {
    ptRemoveCmd(SET_SPEED_PAN, ptDriverSDI);
  }
  else 
    ptRemoveCmd(SET_SPEED_TILT, ptDriverSDI);

  return ETHER;
}


/*------------------------------------------------------------------------
 * SetSpeedProfile
 *
 * Description: 
 *
 * Bilans:      OK 
 */

/* platinetSetSpeedProfile  -  codel EXEC de SetSpeedProfile
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT platineSetSpeedProfile (PLATINE_AXIS_SET_SPEED_PROFILE *param, 
				       int *bilan)
{
  int val;
  double dval;
  PT_CMD_STATUS status;
  PT_ERR err;
 
  /* Test driver init */
  if (!testInit(bilan))
    return ETHER;

  /* TOUT EN MODE BLOQUANT !!! */

  /**
   **  min speed 
   **/
  if (param->unity == PLATINE_RAD) 
    val = PT_RAD_TO_STEP(param->minSpeed,ptDriverSDI);
  else
    val = PT_DEG_TO_STEP(param->minSpeed,ptDriverSDI);

  /* Test */
  if (!testSpeedLimits(param->axis, val)) {
    *bilan = S_platine_INVALID_PARAMS;
    return ETHER;
  }

  /* Envoi la commande */
  if (param->axis == PLATINE_PAN) {
    status = ptSendAndGet(SET_MIN_SPEED_PAN, &val, TRUE, &err, ptDriverSDI);
    if (status != PT_REPLY_OK) {
      *bilan = platineConvertPtErr(err);
      return ETHER;
    }
    /* Mise a jour de la nouvelle valeur */
    SDI_F->stepAxisParams[PLATINE_PAN].minSpeed = val;
    SDI_F->platineParams.speedProfile.pan.minSpeed = PT_STEP_TO_RAD(val,ptDriverSDI);
  }
  else {
    status = ptSendAndGet(SET_MIN_SPEED_TILT, &val, TRUE, &err, ptDriverSDI);
    if (status != PT_REPLY_OK) {
      *bilan = platineConvertPtErr(err);
      return ETHER;
    }
    SDI_F->stepAxisParams[PLATINE_TILT].minSpeed = val;
    SDI_F->platineParams.speedProfile.tilt.minSpeed = PT_STEP_TO_RAD(val,ptDriverSDI);
  }

  /**
   **  max speed 
   **/
  if (param->unity == PLATINE_RAD) 
    val = PT_RAD_TO_STEP(param->maxSpeed,ptDriverSDI);
  else
    val = PT_DEG_TO_STEP(param->maxSpeed,ptDriverSDI);

  /* Test */
  if (!testSpeedLimits(param->axis, val)) {
    *bilan = S_platine_INVALID_PARAMS;
    return ETHER;
  }

  /* Envoi la commande */
  if (param->axis == PLATINE_PAN) {
    status = ptSendAndGet(SET_MAX_SPEED_PAN, &val, TRUE, &err, ptDriverSDI);
    if (status != PT_REPLY_OK) {
      *bilan = platineConvertPtErr(err);
      return ETHER;
    }
    SDI_F->stepAxisParams[PLATINE_PAN].maxSpeed = val;
    SDI_F->platineParams.speedProfile.pan.maxSpeed = PT_STEP_TO_RAD(val,ptDriverSDI);
  }
  else {
    status = ptSendAndGet(SET_MAX_SPEED_TILT, &val, TRUE, &err, ptDriverSDI);
    if (status != PT_REPLY_OK) {
      *bilan = platineConvertPtErr(err);
      return ETHER;
    }
    SDI_F->stepAxisParams[PLATINE_TILT].maxSpeed = val;
    SDI_F->platineParams.speedProfile.tilt.maxSpeed = PT_STEP_TO_RAD(val,ptDriverSDI);
  }

  /**
   **  base speed 
   **/
  if (param->unity == PLATINE_RAD) 
    dval = param->baseSpeed;
  else
    dval = DEG_TO_RAD(param->baseSpeed);

  /* Test */
  if (!testSpeedLimits(param->axis, PT_RAD_TO_STEP(dval,ptDriverSDI))) {
    *bilan = S_platine_INVALID_PARAMS;
    return ETHER;
  }

  /* Envoi la commande */
  if (param->axis == PLATINE_PAN) {
    status = ptSendAndGetRad(SET_BASE_SPEED_PAN, &dval, TRUE, &err, ptDriverSDI);
    if (status != PT_REPLY_OK) {
      *bilan = platineConvertPtErr(err);
      return ETHER;
    }
    SDI_F->platineParams.speedProfile.pan.baseSpeed = dval;
  }
  else {
    status = ptSendAndGetRad(SET_BASE_SPEED_TILT, &dval, TRUE, &err, ptDriverSDI);
    if (status != PT_REPLY_OK) {
      *bilan = platineConvertPtErr(err);
      return ETHER;
    }
    SDI_F->platineParams.speedProfile.tilt.baseSpeed = dval;
  }

  /**
   **   speed 
   **/
  if (param->unity == PLATINE_RAD) 
    val = PT_RAD_TO_STEP(param->speed,ptDriverSDI);
  else
    val = PT_DEG_TO_STEP(param->speed,ptDriverSDI);

  /* Test */
  if (!testSpeed(param->axis, val)) {
    *bilan = S_platine_INVALID_PARAMS;
    return ETHER;
  }

  /* Envoi la commande */
  if (param->axis == PLATINE_PAN) {
    val *= ((double)SDI_F->driverParams.panTrigoRotation);
    status = ptSendAndGet(SET_SPEED_PAN, &val, TRUE, &err, ptDriverSDI);
    if (status != PT_REPLY_OK) {
      *bilan = platineConvertPtErr(err);
      return ETHER;
    }
    SDI_F->platineParams.speedProfile.pan.speed = PT_STEP_TO_RAD(val,ptDriverSDI);
  }
  else {
    status = ptSendAndGet(SET_SPEED_TILT, &val, TRUE, &err, ptDriverSDI);
    if (status != PT_REPLY_OK) {
      *bilan = platineConvertPtErr(err);
      return ETHER;
    }
    SDI_F->platineParams.speedProfile.tilt.speed = PT_STEP_TO_RAD(val,ptDriverSDI);
  }

 
  /**
   **  accel
   **/
  if (param->unity == PLATINE_RAD) 
    val = PT_RAD_TO_STEP(param->accel,ptDriverSDI);
  else
    val = PT_DEG_TO_STEP(param->accel,ptDriverSDI);

  /* Envoi la commande */
  if (param->axis == PLATINE_PAN) {
    status = ptSendAndGet(SET_ACCEL_PAN, &val, TRUE, &err, ptDriverSDI);
    if (status != PT_REPLY_OK) {
      *bilan = platineConvertPtErr(err);
      return ETHER;
    }
    SDI_F->platineParams.speedProfile.pan.accel = PT_STEP_TO_RAD(val,ptDriverSDI);
  }
  else {
    status = ptSendAndGet(SET_ACCEL_TILT, &val, TRUE, &err, ptDriverSDI);
    if (status != PT_REPLY_OK) {
      *bilan = platineConvertPtErr(err);
      return ETHER;
    }
    SDI_F->platineParams.speedProfile.tilt.accel = PT_STEP_TO_RAD(val,ptDriverSDI);
  }

  return ETHER;
}

/*------------------------------------------------------------------------
 * SetPowerModes
 *
 * Description: 
 *
 * Bilans:      OK 
 *              S_platine_INVALID_PARAMS
 */

/* platineSetPowerModes  -  codel EXEC de SetPowerModes
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT platineSetPowerModes (PLATINE_POWER_PARAMS *(power), int *bilan)
{
  PT_CMD_STATUS status;
  PT_ERR err;
  int pipo=0;

  /* Test driver init */
  if (!testInit(bilan))
    return ETHER;

  /* PAN HOLD POWER MODE */
  switch (power->pan.holdPowerMode) {
  case PLATINE_HOLD_POWER_REGULAR:
    status = ptSendAndGet(SET_HOLD_POWER_REGULAR_PAN, &pipo, TRUE, &err, ptDriverSDI);
    break;
  case PLATINE_HOLD_POWER_LOW:
    status = ptSendAndGet(SET_HOLD_POWER_LOW_PAN, &pipo, TRUE, &err, ptDriverSDI);
    break;
  case PLATINE_HOLD_POWER_OFF:
    status = ptSendAndGet(SET_HOLD_POWER_OFF_PAN, &pipo, TRUE, &err, ptDriverSDI);
    break;
  default:
    *bilan = S_platine_INVALID_PARAMS;
    return ETHER;    
  }
  if (status != PT_REPLY_OK) {
    *bilan = platineConvertPtErr(err);
    return ETHER;
  }

  /* PAN MOVE POWER MODE */
  switch (power->pan.movePowerMode) {
  case PLATINE_MOVE_POWER_REGULAR:
    status = ptSendAndGet(SET_MOVE_POWER_REGULAR_PAN, &pipo, TRUE, &err, ptDriverSDI);
    break;
  case PLATINE_MOVE_POWER_LOW:
    status = ptSendAndGet(SET_MOVE_POWER_LOW_PAN, &pipo, TRUE, &err, ptDriverSDI);
    break;
  case PLATINE_MOVE_POWER_HIGH:
    status = ptSendAndGet(SET_MOVE_POWER_HIGH_PAN, &pipo, TRUE, &err, ptDriverSDI);
    break;
  default:
    *bilan = S_platine_INVALID_PARAMS;
    return ETHER;    
  }
  if (status != PT_REPLY_OK) {
    *bilan = platineConvertPtErr(err);
    return ETHER;
  }


  /* TILT HOLD POWER MODE */
  switch (power->tilt.holdPowerMode) {
  case PLATINE_HOLD_POWER_REGULAR:
    status = ptSendAndGet(SET_HOLD_POWER_REGULAR_TILT, &pipo, TRUE, &err, ptDriverSDI);
    break;
  case PLATINE_HOLD_POWER_LOW:
    status = ptSendAndGet(SET_HOLD_POWER_LOW_TILT, &pipo, TRUE, &err, ptDriverSDI);
    break;
  case PLATINE_HOLD_POWER_OFF:
    status = ptSendAndGet(SET_HOLD_POWER_OFF_TILT, &pipo, TRUE, &err, ptDriverSDI);
    break;
  default:
    *bilan = S_platine_INVALID_PARAMS;
    return ETHER;    
  }
  if (status != PT_REPLY_OK) {
    *bilan = platineConvertPtErr(err);
    return ETHER;
  }

  /* TILT MOVE POWER MODE */
  switch (power->tilt.movePowerMode) {
  case PLATINE_MOVE_POWER_REGULAR:
    status = ptSendAndGet(SET_MOVE_POWER_REGULAR_TILT, &pipo, TRUE, &err, ptDriverSDI);
    break;
  case PLATINE_MOVE_POWER_LOW:
    status = ptSendAndGet(SET_MOVE_POWER_LOW_TILT, &pipo, TRUE, &err, ptDriverSDI);
    break;
  case PLATINE_MOVE_POWER_HIGH:
    status = ptSendAndGet(SET_MOVE_POWER_HIGH_TILT, &pipo, TRUE, &err, ptDriverSDI);
    break;
  default:
    *bilan = S_platine_INVALID_PARAMS;
    return ETHER;    
  }
  if (status != PT_REPLY_OK) {
    *bilan = platineConvertPtErr(err);
    return ETHER;
  }
  return ETHER;
}

/*------------------------------------------------------------------------
 * Stop
 *
 * Description: 
 *
 * Bilans:      OK 
 */

/* platineStop  -  codel EXEC de Stop
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT platineStop (int *bilan)
{
  int pipo=0;

  /* Test driver init */
  if (!testInit(bilan))
    return ETHER;

#ifdef OLDVERSION
  ptSendCmdRad(SET_POS_ABS_PAN, SDI_F->states.stateRad.pos.pan + SDI_F->platineParams.offsets.pan, ptDriverSDI);
  ptSendCmdRad(SET_POS_ABS_TILT, SDI_F->states.stateRad.pos.tilt + SDI_F->platineParams.offsets.tilt, ptDriverSDI);
  platineStopStatusP = PT_WAITING_REPLY;
  platineStopStatusT = PT_WAITING_REPLY;
#else
  ptSendCmd(PT_HALT, pipo, ptDriverSDI);
#endif

  return END;
}
ACTIVITY_EVENT platineStopEnd (int *bilan)
{
  PT_ERR err;
  PT_CMD_STATUS status;

#ifdef OLDVERSION
  /* Lecture de l'etat de la commande */
  if (platineStopStatusP == PT_WAITING_REPLY)
    platineStopStatusP = ptGetStatus(SET_POS_ABS_PAN, &err, ptDriverSDI);
  if (platineStopStatusT == PT_WAITING_REPLY)
    platineStopStatusT = ptGetStatus(SET_POS_ABS_TILT, &err, ptDriverSDI);

  /* On attend toujours une reponse */
  if (platineStopStatusP  == PT_WAITING_REPLY || platineStopStatusT == PT_WAITING_REPLY)
    return END;

  /* On a les deux reponses */
  if (platineStopStatusP  != PT_REPLY_OK || platineStopStatusT != PT_REPLY_OK) 
    *bilan = platineConvertPtErr(err);
#else
  status = ptGetStatus(PT_HALT, &err, ptDriverSDI);
/*   printf ("platineStopEnd: status %s\n", ptCmdStatusStrings[status]); */
  if (status == PT_WAITING_REPLY)
    return END;

  if (status != PT_REPLY_OK) {
    printf ("** platineStopEnd: status %s\n", ptCmdStatusStrings[status]);
    *bilan = platineConvertPtErr(err);
  }
#endif

  return ETHER;
}
ACTIVITY_EVENT platineStopInter (int *bilan)
{
#ifdef OLDVERSION
  ptRemoveCmd(SET_POS_ABS_PAN, ptDriverSDI);
  ptRemoveCmd(SET_POS_ABS_TILT, ptDriverSDI);
#else
  ptRemoveCmd(PT_HALT, ptDriverSDI);
#endif
  return ETHER;
}


/*------------------------------------------------------------------------
 * CmdPosPan
 *
 * Description: 
 *
 * Bilans:      OK 
 *              S_platine_INVALID_PARAMS
 */

/* platineMovePosPan  -  codel EXEC de CmdPosPan
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT platineMovePosPanStart (PLATINE_REF_AXIS_POS_STR *(pan), 
				       int *bilan)
{
  double ref;

#ifdef VERBOSE
  printf ("platineMovePosPanStart\n");
#endif

  if (!testInit(bilan))
    return ETHER;

  /* Commande */
  ref = pan->pos;

  /* Conversion en radian */
  if (pan->unity == PLATINE_DEG) 
    ref = DEG_TO_RAD(pan->pos);

  /* Calcul position absolue */
  if (pan->cmdType == PLATINE_ABSOLUTE) {
    /* Offset */
    ref += SDI_F->platineParams.offsets.pan;
  }
  else {
    /* Mvt relatif ( peu prs) */
    ref +=  SDI_F->states.stateRad.pos.pan;
  }

  /* Test validite (bute) */
  if (!testStopEnd(PLATINE_PAN, PT_RAD_TO_STEP(ref,ptDriverSDI))) {
    *bilan = S_platine_INVALID_PARAMS;
    return ETHER;
  }

  /* Enregistre */
  SDI_F->states.stateRad.ref.pan = ref - SDI_F->platineParams.offsets.pan;
  SDI_F->states.stateDeg.ref.pan = RAD_TO_DEG(SDI_F->states.stateRad.ref.pan);

  /* Envoie */
  /* XXX On pourait aussi utiliser la commande de dplacment en relatif
     XXX SET_POS_REL_PAN si PLATINE_RELATIVE,
     XXX mais risque echec surveillance  fin du mvt 
     ATTENTION au ptRemoveCmd(SET_POS_ABS_PAN, ptDriverSDI) EN CAS DE CHANGEMENT
*/
  ptSendCmdRad(SET_POS_ABS_PAN, ((double)SDI_F->driverParams.panTrigoRotation) * ref, ptDriverSDI);

  /* Attend acquitement */
  return EXEC;
}


ACTIVITY_EVENT platineMovePosPanExec (PLATINE_REF_AXIS_POS_STR *(pan), 
				      int *bilan)
{
  PT_ERR err;
  PT_CMD_STATUS status;

  /* Acquitement commande */
  if ((status=ptGetStatus(SET_POS_ABS_PAN, &err, ptDriverSDI)) == PT_WAITING_REPLY)
    return EXEC;

  /* Il y a eu un pb */
  if (status != PT_REPLY_OK && status != PT_NO_CMD) {
    *bilan = platineConvertPtErr(err);
#ifdef VERBOSE
    printf ("platineMovePosPanExec\n");
#endif
    return ETHER;
  }

  /* On n'attend pas la fin */
  if(!pan->survEndFlag){
    ptRemoveCmd(SET_POS_ABS_PAN, ptDriverSDI);
#ifdef VERBOSE
    printf ("platineMovePosPanExec\n");
#endif
    return ETHER;
  }

  /* On attend la fin */
  return END;
}

#define PLATINE_EPSILON_POS 2
ACTIVITY_EVENT platineMovePosPanEnd (PLATINE_REF_AXIS_POS_STR *(pan), 
				     int *bilan)
{
#ifdef VERBOSE
  printf ("platineMovePosPanEnd\n");
#endif
  if (fabs(SDI_F->states.stateRad.ref.pan - SDI_F->states.stateRad.pos.pan) < 
      PT_STEP_TO_RAD(PLATINE_EPSILON_POS,ptDriverSDI)) 
    return ETHER;
  return END;
}

ACTIVITY_EVENT platineMovePosPanInter (PLATINE_REF_AXIS_POS_STR *(pan), 
				       int *bilan)
{
#ifdef VERBOSE
  printf ("platineMovePosPanInter\n");
#endif
  ptRemoveCmd(SET_POS_ABS_PAN, ptDriverSDI);
  return ETHER;
}


/*------------------------------------------------------------------------
 * CmdPosTilt
 *
 * Description: 
 *
 * Bilans:      OK 
 *              S_platine_INVALID_PARAMS
 */

/* platineMovePosTilt  -  codel EXEC de CmdPosTilt
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT platineMovePosTiltStart (PLATINE_REF_AXIS_POS_STR *(tilt), 
				       int *bilan)
{
  double ref;

#ifdef VERBOSE
  printf ("platineMovePosTiltStart\n");
#endif

  /* Test driver init */
  if (!testInit(bilan))
    return ETHER;

  /* Commande */
  ref = tilt->pos;

  /* Conversion en radian */
  if (tilt->unity == PLATINE_DEG) 
    ref = DEG_TO_RAD(tilt->pos);

  /* Calcul position absolue */
  if (tilt->cmdType == PLATINE_ABSOLUTE) {
    /* Offset */
    ref += SDI_F->platineParams.offsets.tilt;
  }
  else {
    /* Mvt relatif ( peu prs) */
    ref +=  SDI_F->states.stateRad.pos.tilt;
  }

  /* Test validite */
  if (!testStopEnd(PLATINE_TILT, PT_RAD_TO_STEP(ref,ptDriverSDI))) {
    *bilan = S_platine_INVALID_PARAMS;
    return ETHER;
  }

  /* Enregistre */
  SDI_F->states.stateRad.ref.tilt = ref - SDI_F->platineParams.offsets.tilt;
  SDI_F->states.stateDeg.ref.tilt = RAD_TO_DEG(SDI_F->states.stateRad.ref.tilt);

  /* Envoie */
  ptSendCmdRad(SET_POS_ABS_TILT, ref, ptDriverSDI);

  /* Attend acquitement */
  return EXEC;
}


ACTIVITY_EVENT platineMovePosTiltExec (PLATINE_REF_AXIS_POS_STR *(tilt), 
				      int *bilan)
{
  PT_ERR err;
  PT_CMD_STATUS status;

  /* Acquitement commande */
  if ((status=ptGetStatus(SET_POS_ABS_TILT, &err, ptDriverSDI)) == PT_WAITING_REPLY)
    return EXEC;

  /* Il y a eu un pb */
  if (status != PT_REPLY_OK && status != PT_NO_CMD) {
    *bilan = platineConvertPtErr(err);
#ifdef VERBOSE
    printf ("platineMovePosTiltExec\n");
#endif
    return ETHER;
  }

  /* On n'attend pas la fin */
  if(!tilt->survEndFlag) {
#ifdef VERBOSE
    printf ("platineMovePosTiltExec\n");
#endif
    return ETHER;
  }

  /* On attend la fin */
  return END;
}
ACTIVITY_EVENT platineMovePosTiltEnd (PLATINE_REF_AXIS_POS_STR *(tilt), 
				     int *bilan)
{
#ifdef VERBOSE
  printf ("platineMovePosTiltEnd\n");
#endif
  if (fabs(SDI_F->states.stateRad.ref.tilt - SDI_F->states.stateRad.pos.tilt) < 
      PT_STEP_TO_RAD(PLATINE_EPSILON_POS,ptDriverSDI))
    return ETHER;
  return END;
}

ACTIVITY_EVENT platineMovePosTiltInter (PLATINE_REF_AXIS_POS_STR *(tilt), 
				       int *bilan)
{
#ifdef VERBOSE
  printf ("platineMovePosTiltInter\n");
#endif
  ptRemoveCmd(SET_POS_ABS_TILT, ptDriverSDI);
  return ETHER;
}


/*------------------------------------------------------------------------
 * CmdPosCoord
 *
 * Description: 
 *
 * Bilans:      OK 
 *              S_platine_INVALID_PARAMS
 */

/* platineMovePosCoord  -  codel EXEC de CmdPosCoord
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT platineMovePosCoordStart (PLATINE_REF_COORD_POS_STR *(coord), 
					 int *bilan)
{
  PLATINE_REF_AXIS_POS_STR axis;

#ifdef VERBOSE
  printf ("platineMovePosCoordStart\n");
#endif

  axis.unity = coord->unity;
  axis.cmdType = coord->cmdType;
  axis.survEndFlag = coord->survEndFlag;

  /* Commande PAN */
  axis.pos = coord->pan;
  if (platineMovePosPanStart(&axis, bilan) != EXEC)
    return ETHER;

  /* Commande TILT */
  axis.pos = coord->tilt;
  if (platineMovePosTiltStart(&axis, bilan) != EXEC) {
    /* XXX OK tant qu'on envoie la commande en absolue */
    ptRemoveCmd(SET_POS_ABS_PAN, ptDriverSDI);
    return ETHER;
  }

  return EXEC;
}

ACTIVITY_EVENT platineMovePosCoord (PLATINE_REF_COORD_POS_STR *(coord), 
				    int *bilan)
{
  PT_ERR errp, errt;
  PT_CMD_STATUS statusp, statust;

  /* Acquitement commande */
  if ((statusp=ptGetStatus(SET_POS_ABS_PAN, &errp, ptDriverSDI)) == PT_WAITING_REPLY ||
      (statust=ptGetStatus(SET_POS_ABS_PAN, &errt, ptDriverSDI)) == PT_WAITING_REPLY)
    return EXEC;
  
  /* Il y a eu un pb */
  if (statusp != PT_REPLY_OK && statusp != PT_NO_CMD) {
    *bilan = platineConvertPtErr(errp);
#ifdef VERBOSE
    printf ("platineMovePosCoord\n");
#endif
    return ETHER;
  }
  if (statust != PT_REPLY_OK && statust != PT_NO_CMD) {
    *bilan = platineConvertPtErr(errt);
#ifdef VERBOSE
    printf ("platineMovePosCoord\n");
#endif
    return ETHER;
  }

  /* On n'attend pas la fin */
  if(!coord->survEndFlag)
    return ETHER;

  /* On attend la fin */
  return END;
}

ACTIVITY_EVENT platineMovePosCoordEnd (PLATINE_REF_COORD_POS_STR *(coord), 
				       int *bilan)
{
#ifdef VERBOSE
  printf ("platineMovePosCoordEnd\n");
#endif
  /* Test si les deux axes sont arrives */
  if (fabs(SDI_F->states.stateRad.ref.pan - SDI_F->states.stateRad.pos.pan) < 
      PT_STEP_TO_RAD(PLATINE_EPSILON_POS,ptDriverSDI) && 
      fabs(SDI_F->states.stateRad.ref.tilt - SDI_F->states.stateRad.pos.tilt) < 
      PT_STEP_TO_RAD(PLATINE_EPSILON_POS,ptDriverSDI))
    return ETHER;
  return END;
}

ACTIVITY_EVENT platineMovePosCoordInter (PLATINE_REF_COORD_POS_STR *(coord), 
					 int *bilan)
{
#ifdef VERBOSE
  printf ("platineMovePosCoordInter\n");
#endif
  ptRemoveCmd(SET_POS_ABS_PAN, ptDriverSDI);
  ptRemoveCmd(SET_POS_ABS_TILT, ptDriverSDI);
  return ETHER;
}

/*------------------------------------------------------------------------
 * SetDefaultPtSettings
 *
 * Description: 
 *
 * Bilans:      OK 
 *              S_platine_INVALID_PARAMS
 */
ACTIVITY_EVENT platineSetDefaultPtSettings(PLATINE_SET_SETTING *set, int *bilan)
{
  PT_CMD cmd;
  PT_ERR err;
  int pipo=0;
  PT_CMD_STATUS status;

  switch(*set) {
  case PLATINE_SAVE_SETTING:
    cmd = SAVE_SETTING;
    break;
  case PLATINE_RESTORE_SAVED_SETTING:
    cmd = RESTORE_SAVED_SETTING;
    break;
  case PLATINE_RESTORE_FACTORY_SETTING:
    cmd = RESTORE_FACTORY_SETTING;
    break;
  default:
    *bilan = S_platine_INVALID_PARAMS;
    return ETHER;
  }

  status = ptSendAndGet(cmd, &pipo, TRUE, &err, ptDriverSDI);
  if (status != PT_REPLY_OK) {
    *bilan = platineConvertPtErr(err);
  }

  /* On relit les parametres */
  SDI_F->initPtParamsFlag = platineInitPtParams();

  return ETHER;
}

/* ====================================================================== */
static BOOL testStopEnd(PLATINE_AXIS axis, int pos)
{
  if (pos > SDI_F->stepAxisParams[axis].posMax ||
      pos < SDI_F->stepAxisParams[axis].posMin)
    return FALSE;
  return TRUE;
}

static BOOL testSpeed(PLATINE_AXIS axis, int speed)
{
  if (speed > SDI_F->stepAxisParams[axis].maxSpeed ||
      speed < SDI_F->stepAxisParams[axis].minSpeed)
    return FALSE;
  return TRUE;
}

static BOOL testSpeedLimits(PLATINE_AXIS axis, int speed)
{
  if (speed >= SDI_F->stepAxisParams[axis].speedMaxMax ||
      speed <= SDI_F->stepAxisParams[axis].speedMinMin)
    return FALSE;
  return TRUE;
}

static BOOL testInit(int *bilan)
{
  if(!SDI_F->initPtParamsFlag) {
    *bilan = S_platine_DRIVER_NOT_INITIALIZED;
    return FALSE;
  }
  return TRUE;
}


/* platineResetPTUdriverExec  -  codel EXEC de ResetPTUdriver
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT platineResetPTUdriverExec (int *bilan)
{
  int pipo=0;

  if (!ptSendBasicCmd(PT_RESET_PTU_CONTROLER, pipo, ptDriverSDI)) {
    *bilan = S_platine_ERROR;
  }
  return ETHER;
}


/*------------------------------------------------------------------------
 * SendAsciiCmd
 *
 * Description: 
 *
 * Bilans:      OK 
 */

/* platineSendAsciiCmdStart  -  codel START de SendAsciiCmd
   Retourne:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT platineSendAsciiCmdStart (GEN_STRING64 *asciiCmd, int *bilan)
{
  if (!ptSendMsg(asciiCmd->name, ptDriverSDI)) {
    *bilan = S_platine_ERROR;
    return ETHER;
  }
  return EXEC;
}

/* platineSendAsciiCmdExec  -  codel EXEC de SendAsciiCmd
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT platineSendAsciiCmdExec (GEN_STRING64 *asciiCmd, int *bilan)
{
  PT_ERR err;
  PT_CMD_STATUS status;

  if ((status=ptGetStatus(PT_ASCII, &err, ptDriverSDI)) == PT_WAITING_REPLY)
    return EXEC;
  if (status != PT_REPLY_OK) 
    *bilan = platineConvertPtErr(err);

  return ETHER;
}

/* platineSendAsciiCmdInter  -  codel INTER de SendAsciiCmd
   Retourne:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT platineSendAsciiCmdInter (GEN_STRING64 *asciiCmd, int *bilan)
{
  ptRemoveCmd(PT_ASCII, ptDriverSDI);
  return ETHER;
}

/*------------------------------------------------------------------------
 * VerboseSwitch
 *
 * Description: 
 *
 * Bilans:      OK 
 */


/* platineVerboseSwicthExec  -  codel EXEC de VerboseSwicth
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT platineVerboseSwitchExec (int *bilan)
{
  if (SDI_F->verbose)
    SDI_F->verbose = FALSE;
  else
    SDI_F->verbose = TRUE;
  ptVerboseSwitch();
  return ETHER;
}



/*------------------------------------------------------------------------
 * SetPlatinePosOnRobot
 *
 * Description: 
 *
 * Reports:      OK
 *              S_platine_NO_POSTER_FOUND
 */

/* PosOnRobotMain  -  codel EXEC of SetPlatinePosOnRobot
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
platineOriginPosOnRobotMain(struct PLATINE_POS_RQST *platineOriginPosOnRobotRqst, 
		      int *report)
{

  /* position given by hand */
  if (!platineOriginPosOnRobotRqst->readOriginFrameByPoster) {
    SDI_F->platineOriginPosOnRobot.sensorToMain.euler = platineOriginPosOnRobotRqst->platineOriginPosOnRobot;
    return ETHER;
  }
  
  /* position given by poster: find out poster */
  if (!platineOriginFramePosterId) {
    if (platinePOM_SENSOR_POSPosterFind(platineOriginPosOnRobotRqst->platineOriginFramePoster, 
					&platineOriginFramePosterId) == ERROR) {
      *report = S_platine_NO_POSTER_FOUND;
      return ETHER;
    }
  }

  /* read pos at least once */
  if (platinePOM_SENSOR_POSPosterRead(platineOriginFramePosterId,
				      &SDI_F->platineOriginPosOnRobot)  
      == ERROR) {
    platineOriginFramePosterId = NULL;
    *report = S_platine_NO_POSTER_FOUND;
    return ETHER;
  } 

  return ETHER;
}



static BOOL
platineConvertTargetToPlatine(PLATINE_CARTESIAN_TARGET *cartesianTarget,
			      double *pan, double *tilt, int *report)
{
  double pos1[3], pos2[3];
  POM_EULER_V originToMain, mainToSensor;
  POM_FRAME currentFrame;
  GEN_POINT_3D camPos;
  GEN_POINT_3D goal;
  double yaw, pitch, dist;

  /* Update platine pos if required */
  if (SDI_F->platineOriginPosOnRobotRqst.readOriginFrameByPoster) {
    platineOriginPosOnRobotMain(&SDI_F->platineOriginPosOnRobotRqst, report);
    if (*report != OK)
      return FALSE;

    /* get pos of robot in origin */
    originToMain = SDI_F->platineOriginPosOnRobot.mainToOrigin;
    mainToSensor = SDI_F->platineOriginPosOnRobot.sensorToMain;
  }

  /* else pos from raw data */
  /* to be done XXXXX */

  /* init pos */
  pos1[0] = cartesianTarget->targetPoint.x;
  pos1[1] = cartesianTarget->targetPoint.y;
  pos1[2] = cartesianTarget->targetPoint.z;
  currentFrame = cartesianTarget->frame;

  /* ORIGIN -> ROBOT */
  if (currentFrame == POM_FRAME_ORIGIN) {
    pomEulerVInvert(&originToMain);
    pomEulerPointAtoB(&originToMain.euler, pos1, pos2); 
    pos1[0] = pos2[0];
    pos1[1] = pos2[1];
    pos1[2] = pos2[2];
    currentFrame = POM_FRAME_ROBOT;
  }
 
  /* ROBOT -> PLATINE */
  if (currentFrame == POM_FRAME_ROBOT) {
    pomEulerVInvert(&mainToSensor);
    pomEulerPointAtoB(&mainToSensor.euler, pos1, pos2);
    pos1[0] = pos2[0];
    pos1[1] = pos2[1];
    pos1[2] = pos2[2];
    currentFrame = POM_FRAME_SENSOR;
  }

  goal.x = pos1[0];
  goal.y = pos1[1];
  goal.z = pos1[2];

  camPos.x = SDI_F->cartesianTarget.sensorOnPlatine.x;
  camPos.y = SDI_F->cartesianTarget.sensorOnPlatine.y;
  camPos.z = SDI_F->cartesianTarget.sensorOnPlatine.z;

  /* MAIN conversion */
  platineCartToPolar(&goal, &camPos, &yaw, &pitch, &dist);

  printf ("dist %g yaw %g pitch %g\n", dist, yaw, pitch);

  *pan = yaw;
  *tilt = pitch;

  return TRUE;
}

/*------------------------------------------------------------------------
 * AimAtTargetPoint
 *
 * Description: 
 *
 * Reports:      OK
 */

static int stayActivelyOnTargetCount;

/* AimAtTargetPointStart  -  codel START of AimAtTargetPoint
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
platineAimAtTargetPointStart(PLATINE_CARTESIAN_TARGET *cartesianTarget,
			     int *report)
{
  double pan, tilt;

  if (!platineConvertTargetToPlatine(cartesianTarget, &pan, &tilt, report))
    return ETHER;

  platineCoordRef.unity = PLATINE_RAD;
  platineCoordRef.cmdType = PLATINE_ABSOLUTE;
  platineCoordRef.pan = pan;
  platineCoordRef.tilt = -tilt;
  platineCoordRef.survEndFlag = FALSE;

  printf ("pan %g tilt %g\n", platineCoordRef.pan, platineCoordRef.tilt);

  stayActivelyOnTargetCount = cartesianTarget->stayActivelyOnTarget;
  return platineMovePosCoordStart(&platineCoordRef, report);
}

/* AimAtTargetPointMain  -  codel EXEC of AimAtTargetPoint
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
platineAimAtTargetPointMain(struct PLATINE_CARTESIAN_TARGET *cartesianTarget, int *report)
{
  platineMovePosCoord(&platineCoordRef, report);

  if (cartesianTarget->stayActivelyOnTarget) {
    double pan, tilt;
    
    /* wait some periods before updating the position */
    if (stayActivelyOnTargetCount > 0) {
      stayActivelyOnTargetCount--;
      printf ("stayActivelyOnTargetCount %d", stayActivelyOnTargetCount);
      return EXEC;
    }
    stayActivelyOnTargetCount = cartesianTarget->stayActivelyOnTarget;
    printf ("stayActivelyOnTargetCount reseted to %d", stayActivelyOnTargetCount);

    /* computes new ref */
    if (!platineConvertTargetToPlatine(cartesianTarget, &pan, &tilt, report))
      return ETHER;

    platineCoordRef.unity = PLATINE_RAD;
    platineCoordRef.cmdType = PLATINE_ABSOLUTE;
    platineCoordRef.pan = pan;
    platineCoordRef.tilt = -tilt;
    platineCoordRef.survEndFlag = FALSE;
    
    printf ("pan %g tilt %g\n", platineCoordRef.pan, platineCoordRef.tilt);

    return platineMovePosCoordStart(&platineCoordRef, report);
  }
  return END;
}

/* AimAtTargetPointEnd  -  codel END of AimAtTargetPoint
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
platineAimAtTargetPointEnd(struct PLATINE_CARTESIAN_TARGET *cartesianTarget, int *report)
{
  return platineMovePosCoordEnd(&platineCoordRef, report);
}

/* AimAtTargetPointInter  -  codel INTER of AimAtTargetPoint
   Returns:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
platineAimAtTargetPointInter(struct PLATINE_CARTESIAN_TARGET *cartesianTarget, int *report)
{
  return platineMovePosCoordInter(&platineCoordRef, report);
}


/*------------------------------------------------------------------------
 * TrackTargetPoint
 *
 * Description: 
 *
 * Reports:      OK
 *              S_platine_NO_POSTER_FOUND
 */


static POSTER_ID trackPointPosterId;
static POM_OBJECT_POS objectPos;

/* TrackTargetPointStart  -  codel START of TrackTargetPoint
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
platineTrackTargetPointStart(struct PLATINE_TRACK_TARGET *trackTarget, int *report)
{
  /* look for poster */
  if (platinePOM_OBJECT_POSPosterFind(trackTarget->platineTargetPosPoster,
				      &trackPointPosterId) == ERROR) {
    *report = S_platine_NO_POSTER_FOUND;
    return ETHER;
  }

  /* read poster */
  if (platinePOM_OBJECT_POSPosterRead(trackPointPosterId, &objectPos) == ERROR) {
    *report = S_platine_NO_POSTER_FOUND;
    trackPointPosterId = NULL;
    return ETHER;
  }
  return EXEC;
}

/* TrackTargetPointMain  -  codel EXEC of TrackTargetPoint
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
platineTrackTargetPointMain(PLATINE_TRACK_TARGET *trackTarget, int *report)
{
  double pan, tilt;
  PLATINE_CARTESIAN_TARGET cartesianTarget;

  /* update target position */
  if (platinePOM_OBJECT_POSPosterRead(trackPointPosterId, &objectPos) == ERROR) {
    *report = S_platine_NO_POSTER_FOUND;
    trackPointPosterId = NULL;
    return ETHER;
  }
  cartesianTarget.frame = objectPos.frame;
  cartesianTarget.targetPoint.x = objectPos.pos.euler.x;
  cartesianTarget.targetPoint.y = objectPos.pos.euler.y;
  cartesianTarget.targetPoint.z = objectPos.pos.euler.z;
  cartesianTarget.sensorOnPlatine = objectPos.sensorPos.sensorToMain.euler;

  /* convert target pos to platine frame */
  if (!platineConvertTargetToPlatine(&cartesianTarget, &pan, &tilt, report))
    return ETHER;

  /* move the platine */
  platineCoordRef.unity = PLATINE_RAD;
  platineCoordRef.cmdType = PLATINE_ABSOLUTE;
  platineCoordRef.pan = pan;
  platineCoordRef.tilt = -tilt;
  platineCoordRef.survEndFlag = FALSE;

  platineMovePosCoordStart(&platineCoordRef, report);
  return EXEC;
}

/* TrackTargetPointEnd  -  codel END of TrackTargetPoint
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
platineTrackTargetPointEnd(struct PLATINE_TRACK_TARGET *trackTarget, int *report)
{
  /* ... add your code here ... */
  return ETHER;
}

/* TrackTargetPointInter  -  codel INTER of TrackTargetPoint
   Returns:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
platineTrackTargetPointInter(struct PLATINE_TRACK_TARGET *trackTarget, int *report)
{
  return platineMovePosCoordInter(&platineCoordRef, report);
}
