/*
 * Copyright (c) 1993-2005,2012 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/**
 ** platineCntrlTaskCodels.c
 **
 ** Codels used by the control task platineCntrlTask
 **
 ** Auteur: 
 ** Date: 
 **
 **/

#include <stdio.h>
#include <string.h>

#include <portLib.h>
#include <h2mathLib.h>

#include "server/platineHeader.h"


/*------------------------------------------------------------------------
 * platineInitCtrl  -  control codel of INIT request InitDriver
 *
 * Description:    
 * Report: OK
 *                 S_platine_INIT_FAILED
 *		   S_platine_INVALID_PARAMS
 *
 * Returns:    OK or ERROR
 */

STATUS
platineInitCtrl(struct PLATINE_DRIVER_PARAMS *driverParams, int *report)
{
  if (driverParams->panTrigoRotation != 1 && driverParams->panTrigoRotation != -1) {
    *report = S_platine_INVALID_PARAMS;
    return ERROR;
  }
  return OK;
}

/*------------------------------------------------------------------------
 * platineConvertParams  -  control codel of CONTROL request GetParams
 *
 * Description:    
 * Report (bilan): OK 
 *
 * Return:    OK ou ERROR
 */

STATUS platineCntrlOffset (PLATINE_SET_PARAM *(param), int *bilan)
{
  double val;

  val = param->val;
  if (param->unity == PLATINE_DEG)
    val = DEG_TO_RAD(val);
  if (param->axis == PLATINE_PAN)
    SDI_F->platineParams.offsets.pan = val;
  else
    SDI_F->platineParams.offsets.tilt = val;
  return OK;
}


STATUS platinePeriodControl (int *(period), int *bilan)
{
  if (*period <= 0) {
    *bilan = S_platine_INVALID_PARAMS;
    return ERROR;
  }
  SDI_F->platineDecount = 0;
  return OK;
}

ACTIVITY_EVENT platineSetOffset (PLATINE_SET_PARAM *(param), int *bilan)
{
  double val;

  /* Unity */
  if (param->unity == PLATINE_RAD)
    val = param->val;
  else
    val = DEG_TO_RAD(param->val);

  if (param->axis == PLATINE_PAN) 
    SDI_F->platineParams.offsets.pan = val;
  else 
    SDI_F->platineParams.offsets.tilt = val;

  return OK;
}

STATUS platineConvertParams (PLATINE_UNITY *unity, int *bilan)
{

  if (*unity == PLATINE_RAD) {
    memcpy(&SDI_F->platineParamsRq, &SDI_F->platineParams, sizeof(PLATINE_PARAMS_STR));
    return OK;
  }

  /* Conversion RAD->DEG */

  /* PAN */
  SDI_F->platineParamsRq.consts.pan.posMin = RAD_TO_DEG(SDI_F->platineParams.consts.pan.posMin);
  SDI_F->platineParamsRq.consts.pan.posMax = RAD_TO_DEG(SDI_F->platineParams.consts.pan.posMax);
  SDI_F->platineParamsRq.consts.pan.speedMinMin = RAD_TO_DEG(SDI_F->platineParams.consts.pan.speedMinMin);
  SDI_F->platineParamsRq.consts.pan.speedMaxMax = RAD_TO_DEG(SDI_F->platineParams.consts.pan.speedMaxMax);
  
  SDI_F->platineParamsRq.offsets.pan = RAD_TO_DEG(SDI_F->platineParams.offsets.pan);
  
  SDI_F->platineParamsRq.speedProfile.pan.minSpeed = RAD_TO_DEG(SDI_F->platineParams.speedProfile.pan.minSpeed);
  SDI_F->platineParamsRq.speedProfile.pan.maxSpeed = RAD_TO_DEG(SDI_F->platineParams.speedProfile.pan.maxSpeed);
  SDI_F->platineParamsRq.speedProfile.pan.baseSpeed = RAD_TO_DEG(SDI_F->platineParams.speedProfile.pan.baseSpeed);
  SDI_F->platineParamsRq.speedProfile.pan.speed = RAD_TO_DEG(SDI_F->platineParams.speedProfile.pan.speed);
  SDI_F->platineParamsRq.speedProfile.pan.accel = RAD_TO_DEG(SDI_F->platineParams.speedProfile.pan.accel);


    SDI_F->platineParamsRq.power.pan.holdPowerMode = SDI_F->platineParams.power.pan.holdPowerMode;
    SDI_F->platineParamsRq.power.pan.movePowerMode = SDI_F->platineParams.power.pan.movePowerMode;

  /* TILT */
  SDI_F->platineParamsRq.consts.tilt.posMin = RAD_TO_DEG(SDI_F->platineParams.consts.tilt.posMin);
  SDI_F->platineParamsRq.consts.tilt.posMax = RAD_TO_DEG(SDI_F->platineParams.consts.tilt.posMax);
  SDI_F->platineParamsRq.consts.tilt.speedMinMin = RAD_TO_DEG(SDI_F->platineParams.consts.tilt.speedMinMin);
  SDI_F->platineParamsRq.consts.tilt.speedMaxMax = RAD_TO_DEG(SDI_F->platineParams.consts.tilt.speedMaxMax);
  
  SDI_F->platineParamsRq.offsets.tilt = RAD_TO_DEG(SDI_F->platineParams.offsets.tilt);
  
  SDI_F->platineParamsRq.speedProfile.tilt.minSpeed = RAD_TO_DEG(SDI_F->platineParams.speedProfile.tilt.minSpeed);
  SDI_F->platineParamsRq.speedProfile.tilt.maxSpeed = RAD_TO_DEG(SDI_F->platineParams.speedProfile.tilt.maxSpeed);
  SDI_F->platineParamsRq.speedProfile.tilt.baseSpeed = RAD_TO_DEG(SDI_F->platineParams.speedProfile.tilt.baseSpeed);
  SDI_F->platineParamsRq.speedProfile.tilt.speed = RAD_TO_DEG(SDI_F->platineParams.speedProfile.tilt.speed);
  SDI_F->platineParamsRq.speedProfile.tilt.accel = RAD_TO_DEG(SDI_F->platineParams.speedProfile.tilt.accel);


    SDI_F->platineParamsRq.power.tilt.holdPowerMode = SDI_F->platineParams.power.tilt.holdPowerMode;
    SDI_F->platineParamsRq.power.tilt.movePowerMode = SDI_F->platineParams.power.tilt.movePowerMode;

    return OK;
}
