/*
 * Copyright (c) 1993-2007,2012 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/**
 ** platineGetPosTaskCodels.c
 **
 ** Codels appeles par la tache platineGetPosTask
 **
 ** Auteur: 
 ** Date: 
 **
 **/

#include <math.h>
#include <stdio.h>
#include <taskLib.h>
#include <string.h>

#include <portLib.h>
#include "h2mathLib.h"

#include "server/platineHeader.h"

#include "driver/ptClient.h"
#include "platineCodels.h"
#include "../platineConst.h"

#include "pomStruct.h"

const POM_EULER_V exportPomSEInit = {
   { 0.0, 0.0, 0.0,     0.0, 0.0, 0.0 },
   { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
          0.0, 0.0, 0.0, 0.0, 0.0,
               0.0, 0.0, 0.0, 0.0,
                    0.0, 0.0, 0.0,
                         0.0, 0.0,
                              0.0 }};


static BOOL  ptResetOn = FALSE;

static int ptPanSpeed;
static int ptTiltSpeed;

static double limitSpeed(PLATINE_AXIS axis, double speed);
static void platineSendSpeedPosRef(PLATINE_AXIS axis, 
				   double speedInRef, 
				   PLATINE_AXIS_SPEED_PROFILE *speedProfile,
				   PLATINE_STEP_AXIS_PARAMS *const,
				   double secureFactor,
				   int securePos,
				   double *refRad,
				   double *refDeg);
static BOOL platineGetAnswer(PT_CMD cmd, int *bilan);

PT_DRIVER_STR *ptDriverSDI; /* Adresse structure driver */

/*------------------------------------------------------------------------
 *
 * platineInitDriverSdi  --  Codel d'initialisation (SDI/f, ...)
 *
 * Description: 
 * 
 * Retourne:    OK ou ERROR
 */

STATUS platineInitDriverSdi(int *bilan)
{

  /* XXXXX REQUETE D"INIT PAS OBLIGATOIRE */
  INIT_RQST = -1;

  /* Default to normal pan orientation */
  SDI_F->driverParams.panTrigoRotation = 1;

  /* Serial port params */
  strcpy(SDI_F->driverParams.deviceName, PLATINE_PORT_NAME);
  SDI_F->driverParams.baudRate = PLATINE_PORT_RATE;

  /* Offsets */
  SDI_F->platineParams.offsets.pan = 0.0;
  SDI_F->platineParams.offsets.tilt = 0.0;
  if (platineParamsPosterWrite(PLATINE_PARAMS_POSTER_ID, 
			      (PLATINE_PARAMS_POSTER_STR *)&SDI_F->platineParams) != OK) {
    h2perror ("platineParamsPosterWrite");
  }
  SDI_F->updatePosterPeriod = 1;
  SDI_F->platineDecount = SDI_F->updatePosterPeriod;
  SDI_F->verbose = FALSE;
  ptDriverSDI = NULL;

  SDI_F->trackPosterId = -1;

  /* Write in poster (init) */
  SDI_F->states.stateRad.pos.pan = 0.;
  SDI_F->states.stateRad.pos.tilt = 0.;
  SDI_F->states.stateDeg.pos.pan = 0.;
  SDI_F->states.stateDeg.pos.tilt = 0.;
  strcpy(SDI_F->exportPomSE.seName, PLATINE_POMSE_NAME);
  SDI_F->exportPomSE.seConfig = exportPomSEInit;
  if (platineStatePosterWrite(PLATINE_STATE_POSTER_ID, 
			      (PLATINE_STATE_POSTER_STR *)&SDI_F->states) != OK) {
    h2perror ("platineStatePosterWrite");
  }
  if (platineSEPosexportPomSEPosterWrite(PLATINE_SEPOS_POSTER_ID, 
					 &SDI_F->exportPomSE) != OK) {
    h2perror ("platineSensorPosPosterWrite");
  }

  printf ("platineInitDriverSdi: position updated each %d period (%dms)\n", 
	  SDI_F->updatePosterPeriod,
	  SDI_F->updatePosterPeriod * PLATINE_TICS_PERIOD * 5);
  /*  SDI_F->initPtParamsFlag = platineInitPtParams();*/

  SDI_F->platineOriginPosOnRobotRqst.readOriginFrameByPoster = 1;

  /* Init platine pos on poster */
  return OK;
}

/*------------------------------------------------------------------------
 * platineInitDriver
 *
 * Description: 
 *
 * Bilans:      OK 
 */

/* platinetReset  -  codel EXEC de Reset
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT platineInitDriver (PLATINE_DRIVER_PARAMS *drivep, int *bilan)
{
  /**
   **  INIT DRIVER
   **/
  if ((ptDriverSDI=ptInitDriver(drivep->deviceName, 
				       drivep->baudRate,
				       SDI_F->verbose)) == NULL) {
    printf ("platineInitDriver: ptInitDriver failed\n");
    *bilan = S_platine_INIT_FAILED;
    SDI_F->initPtParamsFlag = FALSE;
    return ETHER;
  }

  /**
   **  INIT CONSTANTES MODULE
   **/
  SDI_F->initPtParamsFlag = platineInitPtParams();

  if (!SDI_F->initPtParamsFlag) {
    printf ("platineInitDriver: driver cannot init requests\n");
    *bilan = S_platine_INIT_FAILED;
    return ETHER;
  }

  if (platineParamsPosterWrite(PLATINE_PARAMS_POSTER_ID, 
			      (PLATINE_PARAMS_POSTER_STR *)&SDI_F->platineParams) != OK) {
    h2perror ("platineParamsPosterWrite");
  }

  return ETHER;
}


/*------------------------------------------------------------------------
 * Reset
 *
 * Description: 
 *
 * Bilans:      OK 
 */

/* platinetReset  -  codel EXEC de Reset
   Retourne:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT platinetReset (int *bilan)
{
  int pipo=0;
  ptResetOn = TRUE;
  ptSendCmd(PT_RESET, pipo, ptDriverSDI);
  return END;
}

ACTIVITY_EVENT platinetResetEnd (int *bilan)
{
  PT_ERR err;
  PT_CMD_STATUS status;

  if ((status=ptGetStatus(PT_RESET, &err, ptDriverSDI)) == PT_WAITING_REPLY)
    return END;

  if (status != PT_REPLY_OK) 
    *bilan = platineConvertPtErr(err);

  ptResetOn = FALSE;
  return ETHER;
}
ACTIVITY_EVENT platinetResetInter (int *bilan)
{
  PT_ERR err;
  PT_CMD_STATUS status;

  status=ptGetStatus(PT_RESET, &err, ptDriverSDI);
  ptRemoveCmd(PT_RESET, ptDriverSDI);

  if (status != PT_REPLY_OK) 
    *bilan = platineConvertPtErr(err);

  ptResetOn = FALSE;
  return ETHER;  
}

/*------------------------------------------------------------------------
 *
 * platineGetPos  --  codel de l'activite' permanente
 *
 * Description: 
 * 
 * Bilans:      OK 
 * 
 * Retourne:    OK ou ERROR
 */

STATUS platineGetPos(int *bilan)
{
  double pan=0.0, tilt=0.0;
  PT_CMD_STATUS statusp, statust;
  PT_ERR err=0;
/*   static int i; */

/*   i++; */

  if (!SDI_F->initPtParamsFlag || ptResetOn || !SDI_F->updatePosterPeriod)
    return OK;

  (SDI_F->platineDecount)--;
  if (SDI_F->platineDecount>0) {
    return OK;
  }

/*   printf("platineGetPos %d: %d, %d\n", i, SDI_F->platineDecount, SDI_F->updatePosterPeriod); */
  SDI_F->platineDecount = SDI_F->updatePosterPeriod;


  /* Pan pos */
#if 1
  statusp = ptGetSendAndGetRad(GET_POS_ABS_PAN, &pan,  &err, 
			       ptDriverSDI);
#else
  statusp = ptSendAndGetRad(GET_POS_ABS_PAN, &pan, FALSE,
			    &err, ptDriverSDI);
#endif

  pan *= ((double)SDI_F->driverParams.panTrigoRotation);

  /* On a la reponse : mise a jour position 
     (peut-etre pas tres fraiche mais a prendre a tout hazard) */
  if (statusp == PT_REPLY_OK || statusp == PT_WAITING_REPLY) {
    pan -= SDI_F->platineParams.offsets.pan;
    SDI_F->states.stateRad.pos.pan = pan;
    SDI_F->states.stateDeg.pos.pan = RAD_TO_DEG(pan);
  }

  /* Probleme XXXX 
     (pour le moment on se contente de mettre a jour le bilan) */
  else if (statusp != PT_WAITING_REPLY) {
    *bilan = platineConvertPtErr(err);
  }
  
  /* Tilt pos */
#if 1
  statust = ptGetSendAndGetRad(GET_POS_ABS_TILT, &tilt, &err, 
			       ptDriverSDI);
#else
  statust = ptSendAndGetRad(GET_POS_ABS_TILT, &tilt, FALSE,
			    &err, ptDriverSDI);
#endif
  if (statust == PT_REPLY_OK || statust == PT_WAITING_REPLY) {
    tilt -= SDI_F->platineParams.offsets.tilt;
    SDI_F->states.stateRad.pos.tilt = tilt;
    SDI_F->states.stateDeg.pos.tilt = RAD_TO_DEG(tilt);
  }
  /* Probleme XXXX 
     (pour le moment on se contente de mettre a jour le bilan) */
  else if (statust != PT_WAITING_REPLY) {
    *bilan = platineConvertPtErr(err);
  }

  /* mise a jour de la t3d exportPomSE */
  if (statust == PT_REPLY_OK || statust == PT_WAITING_REPLY) {
      SDI_F->exportPomSE.seConfig.euler.yaw = SDI_F->states.stateRad.pos.pan;
      SDI_F->exportPomSE.seConfig.euler.pitch = -SDI_F->states.stateRad.pos.tilt;
      SDI_F->exportPomSE.seConfig.euler.roll = 0.0;
      SDI_F->exportPomSE.seConfig.euler.x = 0.0;
      SDI_F->exportPomSE.seConfig.euler.y = 0.0;
      SDI_F->exportPomSE.seConfig.euler.z = 0.0;
  }
  /* Probleme XXXX 
     (pour le moment on se contente de mettre a jour le bilan) */
  else if (statust != PT_WAITING_REPLY) {
    *bilan = platineConvertPtErr(err);
  }

  /* Write in poster */
  if (statust == PT_REPLY_OK || statust == PT_WAITING_REPLY ||
      statusp== PT_REPLY_OK || statusp == PT_WAITING_REPLY) {
    if (platineStatePosterWrite(PLATINE_STATE_POSTER_ID, 
				(PLATINE_STATE_POSTER_STR *)&SDI_F->states) 
	!= OK) {
      h2perror ("platineStatePosterWrite");
    }

    if (platineSEPosexportPomSEPosterWrite(PLATINE_SEPOS_POSTER_ID, 
					   &SDI_F->exportPomSE) 
	!= OK) {
      h2perror ("platineSensorPosPosterWrite");
    }
  }
  
  return OK;
}

/*------------------------------------------------------------------------
 * TrackPos
 *
 * Description: 
 *
 * Bilans:      OK 
 *              S_platine_INVALID_PARAMS
 */


ACTIVITY_EVENT platineTrackPosStart (GEN_STRING64 *posterName, int *bilan)
{
  if(!SDI_F->initPtParamsFlag) {
    printf ("platineTrackPosStart: driver not initialized \n");
    *bilan = S_platine_DRIVER_NOT_INITIALIZED;
    return ETHER;
  }

  if (posterFind(posterName->name, (POSTER_ID *)&SDI_F->trackPosterId) == ERROR) {
    SDI_F->trackPosterId = -1;
    *bilan = S_platine_POSTER_NOT_FOUND;
    printf ("platineTrackPosStart: poster %s not found\n", posterName->name);
    return ETHER;
  }
  return EXEC;
}

ACTIVITY_EVENT platineTrackPosExec (GEN_STRING64 *posterName, int *bilan)
{
  PLATINE_AXIS_STR ref;
  PT_ERR err=PT_NO_ERR;
  PT_CMD_STATUS statusp, statust;

  /* Lecture reponses platine */
  statusp = ptGetStatus(SET_POS_ABS_PAN, &err, ptDriverSDI);
  if (statusp != PT_WAITING_REPLY && statusp != PT_REPLY_OK 
      && statusp != PT_NO_CMD) {
    *bilan = platineConvertPtErr(err);
    return END;
  }
  statust = ptGetStatus(SET_POS_ABS_TILT, &err, ptDriverSDI);
  if (statust != PT_WAITING_REPLY && statust != PT_REPLY_OK
      && statusp != PT_NO_CMD) {
    *bilan = platineConvertPtErr(err);
    return END;
  }
 
  /* Lecture consigne */
  if (posterRead ((POSTER_ID) SDI_F->trackPosterId, 0, 
		  (void *) &ref, sizeof(ref)) == ERROR) {
    SDI_F->trackPosterId = -1;
    *bilan = S_platine_POSTER_NOT_FOUND;
    return ETHER;
  }

  /* PAN */
  if (ref.pan != SDI_F->states.stateRad.ref.pan) {

    /* Envoie */
    ptSendCmdRad(SET_POS_ABS_PAN, 
		 ((double)SDI_F->driverParams.panTrigoRotation) * (ref.pan + SDI_F->platineParams.offsets.pan), 
		 ptDriverSDI);

    /* Enregistre */
    SDI_F->states.stateRad.ref.pan = ref.pan;
    SDI_F->states.stateDeg.ref.pan = RAD_TO_DEG(ref.pan);
  }

  /* TILT */
  if (ref.tilt != SDI_F->states.stateRad.ref.tilt) {

    /* Envoie */
    ptSendCmdRad(SET_POS_ABS_TILT, 
		 ref.tilt + SDI_F->platineParams.offsets.tilt, 
		 ptDriverSDI);

    /* Enregistre */
    SDI_F->states.stateRad.ref.tilt = ref.tilt;
    SDI_F->states.stateDeg.ref.tilt = RAD_TO_DEG(ref.tilt);
  }

  return EXEC;
}


/*------------------------------------------------------------------------
 * TrackSpeed
 *
 * Description: 
 *
 * Bilans:      OK 
 *              S_platine_INVALID_PARAMS 				*/


ACTIVITY_EVENT platineTrackSpeedStart (GEN_STRING64 *posterName, int *bilan)
{
  PT_ERR err;
#ifdef PT_SPEED_CNTRL_MODE
  int speed=0;
  int pipo=0;
#endif

  if(!SDI_F->initPtParamsFlag) {
    *bilan = S_platine_DRIVER_NOT_INITIALIZED;
    return ETHER;
  }

  /* Acces au poster ? */
  if (posterFind(posterName->name, (POSTER_ID *)&SDI_F->trackPosterId) == ERROR) {
    SDI_F->trackPosterId = -1;
    *bilan = S_platine_POSTER_NOT_FOUND;
    return ETHER;
  }

  /* Sauve les vitesses courantes */
  if (ptSendAndGet(GET_SPEED_PAN, &ptPanSpeed, TRUE, &err, ptDriverSDI) != PT_REPLY_OK) {
    *bilan = platineConvertPtErr(err);
    return ETHER;
  }
  if (ptSendAndGet(GET_SPEED_TILT, &ptTiltSpeed, TRUE, &err, ptDriverSDI) != PT_REPLY_OK) {
    *bilan = platineConvertPtErr(err);
    return ETHER;
  }

  /* CE MODE NE MARCHE PAS */
#ifdef PT_SPEED_CNTRL_MODE
  /* Met les vitesses a 0 */
  if (ptSendAndGet(SET_SPEED_PAN, &speed, TRUE, &err, ptDriverSDI) != PT_REPLY_OK) {
    *bilan = platineConvertPtErr(err);
    return ETHER;
  }
  if (ptSendAndGet(SET_SPEED_TILT, &speed, TRUE, &err, ptDriverSDI) != PT_REPLY_OK) {
    *bilan = platineConvertPtErr(err);
    return ETHER;
  }

  /* Passe en mode commande en vitesse */
  if (ptSendAndGet(PT_VELOCITY_CNTRL, &pipo, TRUE, &err, ptDriverSDI) != PT_REPLY_OK) {
    *bilan = platineConvertPtErr(err);
    return ETHER;
  }
#endif /* PT_SPEED_CNTRL_MODE */

  return EXEC;
}

ACTIVITY_EVENT platineTrackSpeedExec (GEN_STRING64 *posterName, int *bilan)
{
  PLATINE_REF_COORD_SPEED_STR   ref;
  double pmax=120,pmin=-120,tmax=25,tmin=-40;  
  // static int cp=0;

  /* Lecture reponses platine */
  if (!platineGetAnswer(SET_SPEED_PAN, bilan))
    return END;
  if (!platineGetAnswer(SET_SPEED_TILT, bilan))
    return END;
  if (!platineGetAnswer(SET_POS_ABS_PAN, bilan))
    return END;
  if (!platineGetAnswer(SET_POS_ABS_TILT, bilan))
    return END;
  if (!platineGetAnswer(PT_HALT_PAN, bilan))
    return END;
  if (!platineGetAnswer(PT_HALT_TILT, bilan))
    return END;

  /* Lecture consigne */
  if (posterRead ((POSTER_ID) SDI_F->trackPosterId, 0, (void *) &ref, 
		  sizeof(PLATINE_AXIS_STR)) == ERROR) {
    SDI_F->trackPosterId = -1;
    *bilan = S_platine_POSTER_NOT_FOUND;
    return ETHER;
  }
  printf (" P-poster= %.2f , T-poster= %.2f ::",ref.pan,ref.tilt);

   /* PAN*/
   if ((SDI_F->states.stateDeg.pos.pan<=pmax)&(SDI_F->states.stateDeg.pos.pan>=pmin)) {
	printf("P ok -> Pm = %.2f ||",SDI_F->states.stateDeg.pos.pan);

	/* Saturation */
	if((ref.pan>0.5)&(ref.pan<-0.5)) {ref.pan=0; printf ("-- Pan Saturated--");}
   }
   else {ref.pan = 0;printf ("-- Pan limit ");}

  /* TILT */
   if ((SDI_F->states.stateDeg.pos.tilt<=tmax)&(SDI_F->states.stateDeg.pos.tilt>=tmin)) {
	printf("T ok -> Tm = %.2f \n",SDI_F->states.stateDeg.pos.tilt);

        /* Saturation */
	if((ref.tilt>0.5)&(ref.tilt<-0.5)) {ref.tilt=0; printf ("-- Tilt Saturated--\n");}
   }
   else {ref.tilt = 0;printf ("-- Tilt limit \n");}
   
   //test de Track Speed :
	// if (cp < 10) {ref.pan = 0.6; cp++;}  else ref.pan =0; 
  // printf (" cp= &d --->>> P-test= %.2f ::",cp,ref.pan);
	

#ifndef PT_SPEED_CNTRL_MODE

#define PLATINE_SECURE_PAN_FACTOR 0.1
#define PLATINE_MIN_SECURE_PAN_POS 20
  platineSendSpeedPosRef(PLATINE_PAN, 
			 ref.pan, 
			 &SDI_F->platineParams.speedProfile.pan,
			 &SDI_F->stepAxisParams[PLATINE_PAN],
			 PLATINE_SECURE_PAN_FACTOR,
			 PLATINE_MIN_SECURE_PAN_POS,
			 &SDI_F->states.stateRad.ref.pan,
			 &SDI_F->states.stateDeg.ref.pan);

#define PLATINE_SECURE_TILT_FACTOR 0.1
#define PLATINE_MIN_SECURE_TILT_POS 20

  platineSendSpeedPosRef(PLATINE_TILT, 
			 ref.tilt, 
			 &SDI_F->platineParams.speedProfile.tilt,
			 &SDI_F->stepAxisParams[PLATINE_TILT],
			 PLATINE_SECURE_TILT_FACTOR,
			 PLATINE_MIN_SECURE_TILT_POS,
			 &SDI_F->states.stateRad.ref.tilt,
			 &SDI_F->states.stateDeg.ref.tilt);

#else
    /* On change juste les vitesses */
    if (ref.pan != SDI_F->platineParams.speedProfile.pan.speed) {
      SDI_F->platineParams.speedProfile.pan.speed = ref.pan;
      ptSendCmdRad(SET_SPEED_PAN, ((double)SDI_F->driverParams.panTrigoRotation) * ref.pan, ptDriverSDI);
    }
    if (ref.tilt != SDI_F->platineParams.speedProfile.tilt.speed) {
      SDI_F->platineParams.speedProfile.tilt.speed = ref.tilt;
      ptSendCmdRad(SET_SPEED_TILT, ref.tilt, ptDriverSDI);
    }
#endif /* PT_SPEED_CNTRL_MODE */


  return EXEC;
}



static void platineSendSpeedPosRef(PLATINE_AXIS axis, 
				   double speedInRef, 
				   PLATINE_AXIS_SPEED_PROFILE *speedProfile,
				   PLATINE_STEP_AXIS_PARAMS *consts,
				   double secureFactor,
				   int securePos,
				   double *refRad,
				   double *refDeg)
{
  int marge;
  double speedRef, posRef;

  /** 
   ** CALCUL CONSIGNES
   **/

  speedRef = speedInRef;

  /* Calcul consigne position */
  
  /* Calcul distance de scurit pour ne pas atteindre les butes softs 
     car ca bloque tout 
     Avec les valeurs par defaut (1000pas/s et 2000pas/s2) ca donne 25 pas 
     (ie, 1.3 degre) */
  marge = PT_RAD_TO_STEP(secureFactor * 
			 SQR(speedRef) / speedProfile->accel
			 / 2.0, ptDriverSDI);
  if (marge < securePos)
    marge = securePos;

/*   if (speedRef == 0.0)  */
/*     posRef = PT_RAD_TO_STEP(SDI_F->states.stateRad.pos.pan, ptDriverSDI); */
/*   else */ if (speedRef < 0) {
    posRef = consts->posMin + marge;
    speedRef = -speedRef;
  }
  else
    posRef = consts->posMax - marge;

  /* Calcul consigne vitesse */
  speedRef = limitSpeed(axis, speedRef);

  /* La consigne n'a pas change: on s'arrete */
  if (speedProfile->speed == speedRef)
    return;

  /** 
   ** ENVOI CONSIGNES
   **/

  /* Vitesse nulle */
  if(speedRef == 0) {
    speedProfile->speed = speedRef;

    /* XXX aprs un H, la platine refuse de bouger. Je pense que c'est
       parce qu'elle n'a plus de consigne courante et qon ne la renvoie
       pas si le mouvement reprend dans la mme direction qu'avant
       l'interruption */
    /* XXX Il faudrait mettre la position courante */
    *refRad = 0;
    *refDeg = 0;

    if (axis == PLATINE_PAN)
      ptSendCmd(PT_HALT_PAN, posRef, ptDriverSDI);
    else
      ptSendCmd(PT_HALT_TILT, posRef, ptDriverSDI);
    return;
  }

  /* Vitesse non-null */

  /* Ralentissement: on change d'abord la vitesse */
  if (speedRef < speedProfile->speed) {
    speedProfile->speed = speedRef;
    if (axis == PLATINE_PAN)
      ptSendCmdRad(SET_SPEED_PAN, ((double)SDI_F->driverParams.panTrigoRotation) * speedRef, ptDriverSDI);
    else
      ptSendCmdRad(SET_SPEED_TILT, speedRef, ptDriverSDI);
  }

  /* Change la position */
  if (posRef != PT_RAD_TO_STEP(*refRad, ptDriverSDI)) {
    *refRad = PT_STEP_TO_RAD(posRef,ptDriverSDI);
    *refDeg = PT_STEP_TO_DEG(posRef,ptDriverSDI);
    if (axis == PLATINE_PAN)
      ptSendCmd(SET_POS_ABS_PAN, ((double)SDI_F->driverParams.panTrigoRotation) * posRef, ptDriverSDI);
    else
      ptSendCmd(SET_POS_ABS_TILT, posRef, ptDriverSDI);
  }
  
  /* On change la vitesse */
  if (speedRef != speedProfile->speed) {
    speedProfile->speed = speedRef;
    if (axis == PLATINE_PAN)
      ptSendCmdRad(SET_SPEED_PAN, ((double)SDI_F->driverParams.panTrigoRotation) * speedRef, ptDriverSDI);
    else
      ptSendCmdRad(SET_SPEED_TILT, speedRef, ptDriverSDI);
  }
}

ACTIVITY_EVENT platineTrackSpeedInter (GEN_STRING64 *posterName, int *bilan)
{
  PT_ERR err;
#ifdef PT_SPEED_CNTRL_MODE
  int pipo;
#endif

#ifndef PT_SPEED_CNTRL_MODE
  ptSendCmdRad(SET_POS_ABS_PAN, 
	       ((double)SDI_F->driverParams.panTrigoRotation) * (SDI_F->states.stateRad.pos.pan
					+ SDI_F->platineParams.offsets.pan), 
	       ptDriverSDI);
  ptSendCmdRad(SET_POS_ABS_TILT, SDI_F->states.stateRad.pos.tilt 
	       + SDI_F->platineParams.offsets.tilt, ptDriverSDI);
#else

  /* XXXX Arret du mouvement XXX C'est ce qu'on veut ? */
  if (ptSendAndGet(PT_HALT, &pipo, TRUE, &err, ptDriverSDI) != PT_REPLY_OK) 
    *bilan = platineConvertPtErr(err);

  /* Revient en mode commande en position */
  if (ptSendAndGet(PT_POSITION_CNTRL, &pipo, TRUE, &err, ptDriverSDI) != PT_REPLY_OK) {
    *bilan = platineConvertPtErr(err);
    return ETHER;
  }
#endif /* PT_SPEED_CNTRL_MODE */

  /* Remet les vitesses courantes */
  ptPanSpeed *= ((double)SDI_F->driverParams.panTrigoRotation);
  if (ptSendAndGet(SET_SPEED_PAN, &ptPanSpeed, TRUE, &err, ptDriverSDI) != PT_REPLY_OK) {
    *bilan = platineConvertPtErr(err);
    return ETHER;
  }
  if (ptSendAndGet(SET_SPEED_TILT, &ptTiltSpeed, TRUE, &err, ptDriverSDI) != PT_REPLY_OK) {
    *bilan = platineConvertPtErr(err);
    return ETHER;
  }

  return ETHER;
}

/* ====================================================================== */

static double limitSpeed(PLATINE_AXIS axis, double speed)
{
  int fabsSpeed = PT_RAD_TO_STEP(fabs(speed), ptDriverSDI);

  if (axis == PLATINE_PAN) {
    if (fabsSpeed >= SDI_F->stepAxisParams[PLATINE_PAN].maxSpeed)
      return SIGN(speed) * 
	PT_STEP_TO_RAD(SDI_F->stepAxisParams[PLATINE_PAN].maxSpeed-1,ptDriverSDI);
    if (fabsSpeed == 0.0)
      return 0.0;
    if (fabsSpeed <= SDI_F->stepAxisParams[PLATINE_PAN].minSpeed)
      return 0.0;
    return speed;
  }

  if (axis == PLATINE_TILT) {
    if (fabsSpeed >= SDI_F->stepAxisParams[PLATINE_TILT].maxSpeed)
      return SIGN(speed) * 
	PT_STEP_TO_RAD(SDI_F->stepAxisParams[PLATINE_TILT].maxSpeed-1,ptDriverSDI);
    if (fabsSpeed == 0.0)
      return 0.0;
    if (fabsSpeed <= SDI_F->stepAxisParams[PLATINE_TILT].minSpeed)
      return 0.0;
    return speed;
  }

  return speed;
}

/**
 **  platineGetAnswer
 **
 **
 **/
static BOOL platineGetAnswer(PT_CMD cmd, int *bilan)
{
  PT_ERR err;
  PT_CMD_STATUS status;
  
  status = ptGetStatus(SET_SPEED_PAN, &err, ptDriverSDI);
  if (status != PT_WAITING_REPLY && status != PT_REPLY_OK
      && status != PT_NO_CMD) {
    *bilan = platineConvertPtErr(err);
    return FALSE;
  }
  return TRUE;
}
