/**
 ** ptDriverPrivate.h
 ** 
 ** ATTENTION :  comme son nom l'indique,
 **              UTILISE' EXCLUSIVEMENT PAR  ptDriver.c et ptClient.c
 **
 **/


#ifndef PT_READER_H
#define PT_READER_H


/*----------------------------------------------------------------------*/
/* DEFINITION DE TYPES */

/* Type de donnees receptionnees pour les commandes interpretees */
typedef enum PT_DATA_TYPE {
  PT_NO_DATA,
  PT_INT_DATA,
  PT_DOUBLE_DATA_S,  /* Pour indiquer que le %f est en debut */
  PT_STRING_DATA,
  PT_STRING_NO_DATA,
  PT_UNKNOWN_DATA,  /* Il y a une reponse mais on ne connait pas son format */
  PT_NO_ANSWER      /* Il n'y a aucun retour du driver */
} PT_DATA_TYPE;

#define PT_MAX_STRING 64

/* Structure de recuperation des valeurs */
typedef union PT_DATA_VAL {
  int ival;
  double dval;
  char sval[PT_MAX_STRING];
} PT_DATA_VAL;

/* Description des commandes interpretees */
typedef struct PT_CMD_DESC {
  PT_CMD num;                   /* Numero de la commande */
  char *name;                   /* Nom de la commande (pour ptReader) */
  char *inputFormat;            /* Format ASCII pour envoyer une commande */
  PT_DATA_TYPE inputDataType;   /* Type de donnee a envoyer */
  char *outputFormat;           /* Format ASCII d'une reponse */
  PT_DATA_TYPE outputDataType;  /* Type de donnee recuperee */
} PT_CMD_DESC;



/*----------------------------------------------------------------------
 *    DESCRIPTION DES COMMANDES INTERPRETEES 
 *
 *    (Si modif, maintenir coherence avec l'enum PT_CMD dans ptDriver.h)
 */

#define PT_DEFAULT_VERSION_V0 1
#define PT_DEFAULT_VERSION_V1 10
#define PT_DEFAULT_VERSION_V2 11

#define PT_DEFAULT_CMD_DESCRIPTION  PT_CMD_DESCRIPTION_V_1_10;

/* MESSAGES FOR FIRMWARE VERION V1.9 */

#define PT_CMD_DESCRIPTION_V_1_9 { \
  /**** PAN ****/ \
  {GET_POS_ABS_PAN, "GET_POS_ABS_PAN", \
    "pp", PT_NO_DATA, "Current Pan position is %d", PT_INT_DATA}, \
  {GET_POS_REL_PAN, "GET_POS_REL_PAN", \
	"po", PT_NO_DATA, "Current Pan position is %d", PT_INT_DATA}, \
  {GET_SPEED_PAN, "GET_SPEED_PAN", \
	"ps", PT_NO_DATA, "Target Pan speed is %d", PT_INT_DATA}, \
  {GET_ACCEL_PAN, "GET_ACCEL_PAN", \
	"pa", PT_NO_DATA, "Pan acceleration is %d", PT_INT_DATA}, \
\
  {GET_BASE_SPEED_PAN, "GET_BASE_SPEED_PAN", \
        "pb", PT_NO_DATA, "Current Pan base speed is %d", PT_INT_DATA}, \
  {GET_MIN_SPEED_PAN, "GET_MIN_SPEED_PAN", \
	"pl", PT_NO_DATA, "Minimum Pan speed is %d", PT_INT_DATA}, \
  {GET_MAX_SPEED_PAN, "GET_MAX_SPEED_PAN", \
	"pu", PT_NO_DATA, "Maximum Pan speed is %d", PT_INT_DATA}, \
\
  {GET_MIN_POS_PAN, "GET_MIN_POS_PAN", \
	"pn", PT_NO_DATA, "Minimum Pan position is %d", PT_INT_DATA}, \
  {GET_MAX_POS_PAN, "GET_MAX_POS_PAN", \
	"px", PT_NO_DATA, "Maximum Pan position is %d", PT_INT_DATA}, \
  {GET_RES_PAN, "GET_RES_PAN", \
	"pr", PT_NO_DATA, "seconds arc", PT_DOUBLE_DATA_S}, \
\
  {SET_POS_ABS_PAN, "SET_POS_ABS_PAN", \
	"pp%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_POS_REL_PAN, "SET_POS_REL_PAN", \
	"po%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_SPEED_PAN, "SET_SPEED_PAN", \
	"ps%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_ACCEL_PAN, "SET_ACCEL_PAN", \
	"pa%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
\
  {SET_BASE_SPEED_PAN, "SET_BASE_SPEED_PAN", \
	"pb%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_MIN_SPEED_PAN, "SET_MIN_SPEED_PAN", \
	"pl%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_MAX_SPEED_PAN, "SET_MAX_SPEED_PAN", \
	"pu%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  /**** TILT ****/ \
  {GET_POS_ABS_TILT, "GET_POS_ABS_TILT", \
	"tp", PT_NO_DATA, "Current Tilt position is %d", PT_INT_DATA}, \
  {GET_POS_REL_TILT, "GET_POS_REL_TILT", \
	"to", PT_NO_DATA, "Current Tilt position is %d", PT_INT_DATA}, \
  {GET_SPEED_TILT, "GET_SPEED_TILT", \
	"ts", PT_NO_DATA, "Target Tilt speed is %d", PT_INT_DATA}, \
  {GET_ACCEL_TILT, "GET_ACCEL_TILT", \
	"ta", PT_NO_DATA, "Tilt acceleration is %d", PT_INT_DATA}, \
\
  {GET_BASE_SPEED_TILT, "GET_BASE_SPEED_TILT", \
        "tb", PT_NO_DATA, "Current Tilt base speed is %d", PT_INT_DATA}, \
  {GET_MIN_SPEED_TILT, "GET_MIN_SPEED_TILT", \
	"tl", PT_NO_DATA, "Minimum Tilt speed is %d", PT_INT_DATA}, \
  {GET_MAX_SPEED_TILT, "GET_MAX_SPEED_TILT", \
	"tu", PT_NO_DATA, "Maximum Tilt speed is %d", PT_INT_DATA}, \
\
  {GET_MIN_POS_TILT, "GET_MIN_POS_TILT", \
	"tn", PT_NO_DATA, "Minimum Tilt position is %d", PT_INT_DATA}, \
  {GET_MAX_POS_TILT, "GET_MAX_POS_TILT", \
	"tx", PT_NO_DATA, "Maximum Tilt position is %d", PT_INT_DATA}, \
  {GET_RES_TILT, "GET_RES_TILT", \
	"tr", PT_NO_DATA, "seconds arc", PT_DOUBLE_DATA_S}, \
\
  {SET_POS_ABS_TILT, "SET_POS_ABS_TILT", \
	"tp%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_POS_REL_TILT, "SET_POS_REL_TILT", \
	"to%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_SPEED_TILT, "SET_SPEED_TILT", \
	"ts%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_ACCEL_TILT, "SET_ACCEL_TILT", \
	"ta%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
\
  {SET_BASE_SPEED_TILT, "SET_BASE_SPEED_TILT", \
	"tb%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_MIN_SPEED_TILT, "SET_MIN_SPEED_TILT", \
	"tl%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_MAX_SPEED_TILT, "SET_MAX_SPEED_TILT", \
	"tu%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
\
  /**** OTHER ****/ \
  {PT_RESET, "PT_RESET", \
	"r", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {PT_SYNCHO_MODE, "PT_SYNCHO_MODE", \
	"s", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {PT_IMMEDIAT_MODE, "PT_IMMEDIAT_MODE", \
	"i", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {PT_GO_SYNCH_OR_WAIT_IMM, "PT_GO_SYNCH_OR_WAIT_IMM", \
	"a", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {PT_GET_ECHO, "PT_GET_ECHO", \
	"e", PT_NO_DATA, NULL, PT_UNKNOWN_DATA}, \
  {PT_ECHO_ON, "PT_ECHO_ON", \
	"ee", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {PT_ECHO_OFF, "PT_ECHO_OFF", \
	"ed", PT_NO_DATA, NULL, PT_UNKNOWN_DATA}, \
\
  {PT_GET_LIMIT, "PT_GET_LIMIT", \
	"l", PT_NO_DATA, "Limit bounds are %s", PT_STRING_DATA}, \
  {PT_LIMIT_ON, "PT_LIMIT_ON", \
	"le", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {PT_LIMIT_OFF, "PT_LIMIT_OFF", \
	"ld", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {PT_VERSION, "PT_VERSION", \
	"v", PT_NO_DATA, "Pan-Tilt Controller %s", PT_STRING_DATA}, \
\
  {GET_HOLD_POWER_PAN, "GET_HOLD_POWER_PAN", \
	"ph", PT_NO_DATA, "Pan in %s hold", PT_STRING_DATA}, \
  {SET_HOLD_POWER_REGULAR_PAN, "SET_HOLD_POWER_REGULAR_PAN", \
	"phr", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {SET_HOLD_POWER_LOW_PAN, "SET_HOLD_POWER_LOW_PAN", \
	"phl", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {SET_HOLD_POWER_OFF_PAN, "SET_HOLD_POWER_OFF_PAN", \
	"pho", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {GET_MOVE_POWER_PAN, "GET_MOVE_POWER_PAN", \
	"pm", PT_NO_DATA, "Pan in %s move", PT_STRING_DATA}, \
  {SET_MOVE_POWER_REGULAR_PAN, "SET_MOVE_POWER_REGULAR_PAN", \
	"pmr", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {SET_MOVE_POWER_LOW_PAN, "SET_MOVE_POWER_LOW_PAN", \
	"pml", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {SET_MOVE_POWER_HIGH_PAN, "SET_MOVE_POWER_HIGH_PAN", \
	"pmh", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {GET_HOLD_POWER_TILT, "GET_HOLD_POWER_TILT", \
	"th", PT_NO_DATA, "Tilt in %s hold", PT_STRING_DATA}, \
  {SET_HOLD_POWER_REGULAR_TILT, "SET_HOLD_POWER_REGULAR_TILT", \
	"thr", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {SET_HOLD_POWER_LOW_TILT, "SET_HOLD_POWER_LOW_TILT", \
	"thl", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {SET_HOLD_POWER_OFF_TILT, "SET_HOLD_POWER_OFF_TILT", \
	"tho", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {GET_MOVE_POWER_TILT, "GET_MOVE_POWER_TILT", \
	"tm", PT_NO_DATA, "Tilt in %s move", PT_STRING_DATA}, \
  {SET_MOVE_POWER_REGULAR_TILT, "SET_MOVE_POWER_REGULAR_TILT", \
	"tmr", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {SET_MOVE_POWER_LOW_TILT, "SET_MOVE_POWER_LOW_TILT", \
	"tml", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {SET_MOVE_POWER_HIGH_TILT, "SET_MOVE_POWER_HIGH_TILT", \
	"tmh", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {SAVE_SETTING, "SAVE_SETTING", \
	"ds", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {RESTORE_SAVED_SETTING, "RESTORE_SAVED_SETTING", \
	"dr", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {RESTORE_FACTORY_SETTING, "RESTORE_FACTORY_SETTING", \
	"df", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {PT_ASCII, "PT_ASCII", NULL, PT_NO_DATA, NULL, PT_UNKNOWN_DATA}, \
\
  {PT_HELP, "PT_HELP", \
	"?", PT_NO_DATA, NULL, PT_UNKNOWN_DATA}, \
\
\
  {PT_HALT, "PT_HALT", \
	"h", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {PT_HALT_PAN, "PT_HALT_PAN", \
	"hp", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {PT_HALT_TILT, "PT_HALT_TILT", \
	"ht", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {PT_GET_CNTRL_MODE, "PT_GET_CNTRL_MODE", \
	"c", PT_NO_DATA, "%s %s speed control mode", PT_STRING_DATA}, \
  {PT_VELOCITY_CNTRL, "PT_VELOCITY_CNTRL", \
	"cv", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {PT_POSITION_CNTRL, "PT_POSITION_CNTRL", \
	"ci", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {PT_RESET_PAN, "PT_RESET_PAN", \
	"rp", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {PT_RESET_TILT, "PT_RESET_TILT", \
	"rt", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {PT_RESET_PTU_CONTROLER, "PT_RESET_PTU_CONTROLER", \
	"   ", PT_NO_DATA, NULL, PT_NO_ANSWER} \
} /*  PT_CMD_DESCRIPTION_V_1_9 */


/* MESSAGES FOR FIRMWARE VERION V1.10 */

#define PT_CMD_DESCRIPTION_V_1_10 { \
  /**** PAN ****/ \
  {GET_POS_ABS_PAN, "GET_POS_ABS_PAN", \
    "pp", PT_NO_DATA, "Current Pan position is %d", PT_INT_DATA}, \
  {GET_POS_REL_PAN, "GET_POS_REL_PAN", \
	"po", PT_NO_DATA, "Current Pan position is %d", PT_INT_DATA}, \
  {GET_SPEED_PAN, "GET_SPEED_PAN", \
	"ps", PT_NO_DATA, "Target Pan speed is %d", PT_INT_DATA}, \
  {GET_ACCEL_PAN, "GET_ACCEL_PAN", \
	"pa", PT_NO_DATA, "Pan acceleration is %d", PT_INT_DATA}, \
\
  {GET_BASE_SPEED_PAN, "GET_BASE_SPEED_PAN", \
        "pb", PT_NO_DATA, "Base Pan speed is %d", PT_INT_DATA}, \
  {GET_MIN_SPEED_PAN, "GET_MIN_SPEED_PAN", \
	"pl", PT_NO_DATA, "Minimum Pan speed is %d", PT_INT_DATA}, \
  {GET_MAX_SPEED_PAN, "GET_MAX_SPEED_PAN", \
	"pu", PT_NO_DATA, "Maximum Pan speed is %d", PT_INT_DATA}, \
\
  {GET_MIN_POS_PAN, "GET_MIN_POS_PAN", \
	"pn", PT_NO_DATA, "Minimum Pan position is %d", PT_INT_DATA}, \
  {GET_MAX_POS_PAN, "GET_MAX_POS_PAN", \
	"px", PT_NO_DATA, "Maximum Pan position is %d", PT_INT_DATA}, \
  {GET_RES_PAN, "GET_RES_PAN", \
	"pr", PT_NO_DATA, "seconds arc", PT_DOUBLE_DATA_S}, \
\
  {SET_POS_ABS_PAN, "SET_POS_ABS_PAN", \
	"pp%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_POS_REL_PAN, "SET_POS_REL_PAN", \
	"po%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_SPEED_PAN, "SET_SPEED_PAN", \
	"ps%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_ACCEL_PAN, "SET_ACCEL_PAN", \
	"pa%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
\
  {SET_BASE_SPEED_PAN, "SET_BASE_SPEED_PAN", \
	"pb%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_MIN_SPEED_PAN, "SET_MIN_SPEED_PAN", \
	"pl%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_MAX_SPEED_PAN, "SET_MAX_SPEED_PAN", \
	"pu%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  /**** TILT ****/ \
  {GET_POS_ABS_TILT, "GET_POS_ABS_TILT", \
	"tp", PT_NO_DATA, "Current Tilt position is %d", PT_INT_DATA}, \
  {GET_POS_REL_TILT, "GET_POS_REL_TILT", \
	"to", PT_NO_DATA, "Current Tilt position is %d", PT_INT_DATA}, \
  {GET_SPEED_TILT, "GET_SPEED_TILT", \
	"ts", PT_NO_DATA, "Target Tilt speed is %d", PT_INT_DATA}, \
  {GET_ACCEL_TILT, "GET_ACCEL_TILT", \
	"ta", PT_NO_DATA, "Tilt acceleration is %d", PT_INT_DATA}, \
\
  {GET_BASE_SPEED_TILT, "GET_BASE_SPEED_TILT", \
        "tb", PT_NO_DATA, "Base Tilt speed is %d", PT_INT_DATA}, \
  {GET_MIN_SPEED_TILT, "GET_MIN_SPEED_TILT", \
	"tl", PT_NO_DATA, "Minimum Tilt speed is %d", PT_INT_DATA}, \
  {GET_MAX_SPEED_TILT, "GET_MAX_SPEED_TILT", \
	"tu", PT_NO_DATA, "Maximum Tilt speed is %d", PT_INT_DATA}, \
\
  {GET_MIN_POS_TILT, "GET_MIN_POS_TILT", \
	"tn", PT_NO_DATA, "Minimum Tilt position is %d", PT_INT_DATA}, \
  {GET_MAX_POS_TILT, "GET_MAX_POS_TILT", \
	"tx", PT_NO_DATA, "Maximum Tilt position is %d", PT_INT_DATA}, \
  {GET_RES_TILT, "GET_RES_TILT", \
	"tr", PT_NO_DATA, "seconds arc", PT_DOUBLE_DATA_S}, \
\
  {SET_POS_ABS_TILT, "SET_POS_ABS_TILT", \
	"tp%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_POS_REL_TILT, "SET_POS_REL_TILT", \
	"to%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_SPEED_TILT, "SET_SPEED_TILT", \
	"ts%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_ACCEL_TILT, "SET_ACCEL_TILT", \
	"ta%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
\
  {SET_BASE_SPEED_TILT, "SET_BASE_SPEED_TILT", \
	"tb%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_MIN_SPEED_TILT, "SET_MIN_SPEED_TILT", \
	"tl%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
  {SET_MAX_SPEED_TILT, "SET_MAX_SPEED_TILT", \
	"tu%d", PT_INT_DATA, NULL, PT_NO_DATA}, \
\
  /**** OTHER ****/ \
  {PT_RESET, "PT_RESET", \
	"r", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {PT_SYNCHO_MODE, "PT_SYNCHO_MODE", \
	"s", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {PT_IMMEDIAT_MODE, "PT_IMMEDIAT_MODE", \
	"i", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {PT_GO_SYNCH_OR_WAIT_IMM, "PT_GO_SYNCH_OR_WAIT_IMM", \
	"a", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {PT_GET_ECHO, "PT_GET_ECHO", \
	"e", PT_NO_DATA, NULL, PT_UNKNOWN_DATA}, \
  {PT_ECHO_ON, "PT_ECHO_ON", \
	"ee", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {PT_ECHO_OFF, "PT_ECHO_OFF", \
	"ed", PT_NO_DATA, NULL, PT_UNKNOWN_DATA}, \
\
  {PT_GET_LIMIT, "PT_GET_LIMIT", \
	"l", PT_NO_DATA, "Limit bounds are %s", PT_STRING_DATA}, \
  {PT_LIMIT_ON, "PT_LIMIT_ON", \
	"le", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {PT_LIMIT_OFF, "PT_LIMIT_OFF", \
	"ld", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {PT_VERSION, "PT_VERSION", \
	"v", PT_NO_DATA, "Pan-Tilt Controller %s", PT_STRING_DATA}, \
\
  {GET_HOLD_POWER_PAN, "GET_HOLD_POWER_PAN", \
	"ph", PT_NO_DATA, "Pan in %s hold", PT_STRING_DATA}, \
  {SET_HOLD_POWER_REGULAR_PAN, "SET_HOLD_POWER_REGULAR_PAN", \
	"phr", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {SET_HOLD_POWER_LOW_PAN, "SET_HOLD_POWER_LOW_PAN", \
	"phl", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {SET_HOLD_POWER_OFF_PAN, "SET_HOLD_POWER_OFF_PAN", \
	"pho", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {GET_MOVE_POWER_PAN, "GET_MOVE_POWER_PAN", \
	"pm", PT_NO_DATA, "Pan in %s move", PT_STRING_DATA}, \
  {SET_MOVE_POWER_REGULAR_PAN, "SET_MOVE_POWER_REGULAR_PAN", \
	"pmr", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {SET_MOVE_POWER_LOW_PAN, "SET_MOVE_POWER_LOW_PAN", \
	"pml", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {SET_MOVE_POWER_HIGH_PAN, "SET_MOVE_POWER_HIGH_PAN", \
	"pmh", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {GET_HOLD_POWER_TILT, "GET_HOLD_POWER_TILT", \
	"th", PT_NO_DATA, "Tilt in %s hold", PT_STRING_DATA}, \
  {SET_HOLD_POWER_REGULAR_TILT, "SET_HOLD_POWER_REGULAR_TILT", \
	"thr", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {SET_HOLD_POWER_LOW_TILT, "SET_HOLD_POWER_LOW_TILT", \
	"thl", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {SET_HOLD_POWER_OFF_TILT, "SET_HOLD_POWER_OFF_TILT", \
	"tho", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {GET_MOVE_POWER_TILT, "GET_MOVE_POWER_TILT", \
	"tm", PT_NO_DATA, "Tilt in %s move", PT_STRING_DATA}, \
  {SET_MOVE_POWER_REGULAR_TILT, "SET_MOVE_POWER_REGULAR_TILT", \
	"tmr", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {SET_MOVE_POWER_LOW_TILT, "SET_MOVE_POWER_LOW_TILT", \
	"tml", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {SET_MOVE_POWER_HIGH_TILT, "SET_MOVE_POWER_HIGH_TILT", \
	"tmh", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {SAVE_SETTING, "SAVE_SETTING", \
	"ds", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {RESTORE_SAVED_SETTING, "RESTORE_SAVED_SETTING", \
	"dr", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {RESTORE_FACTORY_SETTING, "RESTORE_FACTORY_SETTING", \
	"df", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {PT_ASCII, "PT_ASCII", NULL, PT_NO_DATA, NULL, PT_UNKNOWN_DATA}, \
\
  {PT_HELP, "PT_HELP", \
	"?", PT_NO_DATA, NULL, PT_UNKNOWN_DATA}, \
\
\
  {PT_HALT, "PT_HALT", \
	"h", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {PT_HALT_PAN, "PT_HALT_PAN", \
	"hp", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {PT_HALT_TILT, "PT_HALT_TILT", \
	"ht", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {PT_GET_CNTRL_MODE, "PT_GET_CNTRL_MODE", \
	"c", PT_NO_DATA, "%s %s speed control mode", PT_STRING_DATA}, \
  {PT_VELOCITY_CNTRL, "PT_VELOCITY_CNTRL", \
	"cv", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {PT_POSITION_CNTRL, "PT_POSITION_CNTRL", \
	"ci", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {PT_RESET_PAN, "PT_RESET_PAN", \
	"rp", PT_NO_DATA, NULL, PT_NO_DATA}, \
  {PT_RESET_TILT, "PT_RESET_TILT", \
	"rt", PT_NO_DATA, NULL, PT_NO_DATA}, \
\
  {PT_RESET_PTU_CONTROLER, "PT_RESET_PTU_CONTROLER", \
	"   ", PT_NO_DATA, NULL, PT_NO_ANSWER} \
}  /*  PT_CMD_DESCRIPTION_V_1_10 */


/*----------------------------------------------------------------------
 *    DESCRIPTION DES MESSAGES D'ERREUR  (XXX A COMPLETER XXXXX)
 *
 *    (Si modif, maintenir coherence avec l'enum PT_ERR dans ptDriver.h)
 */
typedef struct PT_ERR_FORMAT {
  int num;
  char *name;
  char *format;
} PT_ERR_FORMAT, PT_STRINGS_FORMAT;


/* Nombre d'erreur avec chaine non-null dans ptErrDesc  */
#define PT_NB_ERR_FORMATS  11

#define PT_ERR_DESC { \
  {PT_OUT_OF_RANGE, "PT_OUT_OF_RANGE", "Value out of range"}, \
  {PT_SMALL_PAN_VALUE, "PT_SMALL_PAN_VALUE", "Minimum allowable Pan position is"}, \
  {PT_BIG_PAN_VALUE, "PT_BIG_PAN_VALUE", "Maximum allowable Pan position is"}, \
  {PT_SMALL_TILT_VALUE, "PT_SMALL_TILT_VALUE", "Minimum allowable Tilt position is"}, \
  {PT_BIG_TILT_VALUE, "PT_BIG_TILT_VALUE", "Maximum allowable Tilt position is"}, \
  {PT_SMALL_PAN_SPEED, "PT_SMALL_PAN_SPEED", "Pan speed cannot be less than"}, \
  {PT_SMALL_TILT_SPEED, "PT_SMALL_TILT_SPEED", "Tilt speed cannot be less than"}, \
  {PT_ILLEGAL_COMMAND, "PT_ILLEGAL_COMMAND", "Illegal command entered"}, \
  {PT_ILLEGAL_ACCEL, "PT_ILLEGAL_ACCEL", "Acceleration rate exceeds acceleration table size"}, \
  {PT_SYS_ERR, "PT_SYS_ERR", "SYSTEM ERROR"}, \
  {PT_STOP_ENDS, "PT_STOP_ENDS", "!T!T!P!P"}, \
  {PT_TIMEOUT_ERR, "PT_TIMEOUT_ERR", NULL}, \
  {PT_UNKNOWN_ERR, "PT_UNKNOWN_ERR", NULL}, \
  {PT_NO_ERR, "PT_NO_ERR", NULL} \
}
  


/*----------------------------------------------------------------------*/
/* LES CONSTANTES A INITIALISER */ 

/* Les commandes a envoyer a l'init pour initialiser les constantes */ 
/* GET_RES_PAN et GET_RES_TILT sont imperatifs */ 

#define PT_INIT_GET_CONST { \
  GET_MIN_POS_PAN, \
  GET_MAX_POS_PAN, \
  GET_MIN_POS_TILT, \
  GET_MAX_POS_TILT \
}

/*----------------------------------------------------------------------*/
/* CHAINES A DECODER */
/* Verifier la coherence avec PT_STRINGS_DECODE dans ptDriver.h */

#define PT_STRINGS_DESC { \
  {PT_REGULAR_POWER, "PT_REGULAR_POWER", "REGULAR"}, \
  {PT_LOW_POWER, "PT_LOW_POWER", "LOW"}, \
  {PT_HIGH_POWER, "PT_HIGH_POWER", "HIGH"}, \
  {PT_POWER_OFF, "PT_POWER_OFF", "OFF"}, \
  {PT_ENABLE, "PT_ENABLE", "ENABLE"}, \
  {PT_DISABLE, "PT_DISABLE", "DISABLE"} \
}

/*----------------------------------------------------------------------*/
/* STRUCTURE ETAT D'UNE COMMANDE */

typedef struct PT_CMD_STATE {
  unsigned short nbWaitedReplies;
  unsigned short nbReceivedReplies;
  unsigned short nbRepliesToRemove;
  PT_CMD_STATUS replyStatus;
  PT_ERR replyErr;
  unsigned short timeout;
} PT_CMD_STATE;

/*----------------------------------------------------------------------*/
/* SDI (STRUCTURE COMPLETE) */

typedef struct PT_DRIVER_PRIVATE {

  /* Relatif aux messages */
  char ptMsg[PT_MAX_MSG_LENGTH];  /* Dernier message recu par ptReader */
  int  numDriver;
  int  taskId;                    /* Identificateur de la tache (process) */
  BOOL ptMsgFlag;                 /* Flag pour signaler s'il traine un 
				     message non interprete ni lu par 
				     ailleurs */
  char ptErrMsg[PT_MAX_MSG_LENGTH];  /* Dernier message d'erreur recu par 
					ptReader */
  PT_ERR ptErr;
  int ptSerialFd;               /* Ligne serie ouverte */


  /* Relatif aux commandes */
  PT_CMD_STATE ptCmdState[PT_NB_CMD];
  PT_DATA_VAL ptDataVal[PT_NB_CMD];  /* Valeur retournee par la commande 
					(un peu emcombrant car de nombreuses
					commandes ne retourne pas de valeur: 
					a condenser ou en profiter pour 
					memoriser la valeur envoyee) */
#define PT_MAX_WAITED_REPLIES 100
  PT_CMD ptWaitedRepliesFifoList[PT_MAX_WAITED_REPLIES];  /* FIFO list */
  int ptNbWaitedRepliesFifoList;

  SEM_ID ptSDISem;    /* SEM to protect access of this structure */

} PT_DRIVER_PRIVATE;


#endif
