#ifndef PLATINE_STRUCT_H
#define PLATINE_STRUCT_H

#define PLATINE_MAX_NAME_LENGTH  32

typedef struct PLATINE_DRIVER_PARAMS {
  char deviceName[PLATINE_MAX_NAME_LENGTH];
  int  baudRate;
  int  panTrigoRotation;
} PLATINE_DRIVER_PARAMS;

/** ********************************************************************* **
 **                     STRUCTURES COMMUNES                               **
 ** ********************************************************************* **/

typedef struct PLATINE_AXIS_STR {
  double pan;
  double tilt;
} PLATINE_AXIS_STR;

typedef enum PLATINE_BOOL {
  PLATINE_FALSE,
  PLATINE_TRUE
} PLATINE_BOOL;

typedef enum PLATINE_UNITY {
  PLATINE_RAD,
  PLATINE_DEG
} PLATINE_UNITY;

typedef enum PLATINE_AXIS {
  PLATINE_PAN,
  PLATINE_TILT
} PLATINE_AXIS;

typedef enum PLATINE_CMD_TYPE {
  PLATINE_RELATIVE,
  PLATINE_ABSOLUTE
} PLATINE_CMD_TYPE;

typedef struct PLATINE_3D_POINT {
  double x, y, z;
} PLATINE_3D_POINT;

/** ********************************************************************* **
 **                                ETAT                                   **
 ** ********************************************************************* **/
typedef struct PLATINE_STATE {
  PLATINE_AXIS_STR pos;
  PLATINE_AXIS_STR ref;
} PLATINE_STATE;

typedef struct PLATINE_STATES {
  PLATINE_STATE stateRad;
  PLATINE_STATE stateDeg;
} PLATINE_STATES;


/** ********************************************************************* **
 **                       CONSIGNES EN POSITION                           **
 ** ********************************************************************* **/

/* Consigne position */
typedef struct PLATINE_REF_AXIS_POS_STR {
  PLATINE_UNITY    unity;    /* PLATINE_RAD ou PLATINE_DEG */
  PLATINE_CMD_TYPE cmdType;  /* PLATINE_RELATIVE ou PLATINE_ABSOLUTE */
  double           pos;      /* Angle de consigne */
  PLATINE_BOOL     survEndFlag; /* Surveille arrive avant envoi final reply */
  int              unused;
} PLATINE_REF_AXIS_POS_STR;

/* Mouvement en position coordonne
   (les deux mouvents demarrent en meme temps) */
typedef struct PLATINE_REF_COORD_POS_STR {
  PLATINE_UNITY    unity;    /* PLATINE_RAD ou PLATINE_DEG */
  PLATINE_CMD_TYPE cmdType;  /* PLATINE_RELATIVE ou PLATINE_ABSOLUTE */
  double           pan;      /* Angle de consigne */
  double           tilt;     /* Angle de consigne */
  PLATINE_BOOL     survEndFlag; /* Surveille arrive avant envoi final reply */
  int              unused;
} PLATINE_REF_COORD_POS_STR;

typedef struct PLATINE_REF_POS_STR {
  PLATINE_REF_AXIS_POS_STR   pan;
  PLATINE_REF_AXIS_POS_STR   tilt;
  PLATINE_REF_COORD_POS_STR  coord;
} PLATINE_REF_POS_STR;

typedef struct PLATINE_CARTESIAN_TARGET {
  POM_EULER sensorOnPlatine;  /* Position on the platin of the sensor that looks at the target
				 (0,0,0,0,0,0: center of rotation of the platine) */
  POM_FRAME frame;
  int stayActivelyOnTarget;  /* 0: move the PTU only once toward the target; 
				>0: force the motion to point on target, even if the robot moves, the value is the period */
  PLATINE_3D_POINT targetPoint;
} PLATINE_CARTESIAN_TARGET;

typedef struct PLATINE_TRACK_TARGET {
  char platineTargetPosPoster[PLATINE_MAX_NAME_LENGTH]; /* pos given by this poster */
} PLATINE_TRACK_TARGET;

typedef struct PLATINE_POS_RQST {
  PLATINE_BOOL readOriginFrameByPoster; /* if TRUE read pos by poster, either 
					   using pos data on this request */
  int unused;   /* if static data, to be read only once, 
					either will be read periodically */
  POM_EULER    platineOriginPosOnRobot;  /* the position if given by request */
  char         platineOriginFramePoster[PLATINE_MAX_NAME_LENGTH]; /* pos given by this poster */
} PLATINE_POS_RQST;

/** ********************************************************************* **
 **                       CONSIGNES EN VITESSE                            **
 ** ********************************************************************* **/

/* Consigne vitesse */
typedef struct PLATINE_REF_AXIS_SPEED_STR {
  double        speed;               /* Vitesse de consigne */
  PLATINE_UNITY  unity;              /* PLATINE_RAD ou PLATINE_DEG */
  int           unused;
} PLATINE_REF_AXIS_SPEED_STR;

/* Mouvement en vitesses coordonnees 
   (les deux mouvents demarrent en meme temps) */
typedef struct PLATINE_REF_COORD_SPEED_STR {
  double        pan;                 /* Vitesse de consigne */
  double        tilt;                /* Vitesse de consigne */
  PLATINE_UNITY unity;               /* PLATINE_RAD ou PLATINE_DEG */
  int           unused;
} PLATINE_REF_COORD_SPEED_STR;

/** ********************************************************************* **
 **                  CHANGEMENT PARAMETRES PLATINE                        **
 ** ********************************************************************* **/

typedef struct PLATINE_SET_PARAM {
  double val;
  PLATINE_AXIS axis;       /* PLATINE_PAN ou PLATINE_TILT */
  PLATINE_UNITY unity;     /* PLATINE_RAD ou PLATINE_DEG */
} PLATINE_SET_PARAM;

/** ********************************************************************* **
 **                 PARAMETRES CONSTANTS PLATINE                          **
 ** ********************************************************************* **/

/* Parametres constants (radians) */
typedef struct PLATINE_AXIS_CONST_PARAMS {
  double        posMin;
  double        posMax;
  double        speedMinMin;
  double        speedMaxMax;
} PLATINE_AXIS_CONST_PARAMS;

typedef struct  PLATINE_CONST_PARAMS {
  PLATINE_AXIS_CONST_PARAMS   pan;
  PLATINE_AXIS_CONST_PARAMS   tilt;
} PLATINE_CONST_PARAMS;

typedef struct PLATINE_CONVERT_PARAMS {
  double ptResolution;
  double step2rad;
  double rad2step;
  double step2deg;
  double deg2step;
} PLATINE_CONVERT_PARAMS;

typedef struct PLATINE_STEP_AXIS_PARAMS {
  int  posMin;
  int  posMax;
  int  minSpeed;
  int  maxSpeed;
  int  speedMinMin;
  int  speedMaxMax;
} PLATINE_STEP_AXIS_PARAMS;

/** ********************************************************************* **
 **               PARAMETRES MODIFIABLES PLATINE                          **
 ** ********************************************************************* **/

typedef struct  PLATINE_AXIS_SPEED_PROFILE {
  double        minSpeed;
  double        maxSpeed;
  double        baseSpeed;
  double        speed;
  double        accel;
} PLATINE_AXIS_SPEED_PROFILE;

typedef struct  PLATINE_AXIS_SET_SPEED_PROFILE {
  PLATINE_AXIS  axis;       /* PLATINE_PAN ou PLATINE_TILT */
  PLATINE_UNITY unity;      /* PLATINE_RAD ou PLATINE_DEG */
  double        minSpeed;
  double        maxSpeed;
  double        baseSpeed;
  double        speed;
  double        accel;
} PLATINE_AXIS_SET_SPEED_PROFILE;


typedef struct  PLATINE_SPEED_PROFILE {
  PLATINE_AXIS_SPEED_PROFILE   pan;
  PLATINE_AXIS_SPEED_PROFILE   tilt;
} PLATINE_SPEED_PROFILE;


/** ********************************************************************* **
 **                 PARAMETRES GESTION PUISSANCE                          **
 ** ********************************************************************* **/

typedef enum PLATINE_HOLD_POWER_MODE {
  PLATINE_HOLD_POWER_REGULAR,
  PLATINE_HOLD_POWER_LOW,
  PLATINE_HOLD_POWER_OFF
} PLATINE_HOLD_POWER_MODE;

typedef enum PLATINE_MOVE_POWER_MODE {
  PLATINE_MOVE_POWER_REGULAR,
  PLATINE_MOVE_POWER_LOW,
  PLATINE_MOVE_POWER_HIGH
} PLATINE_MOVE_POWER_MODE;

typedef struct  PLATINE_AXIS_POWER_PARAMS {
  PLATINE_HOLD_POWER_MODE   holdPowerMode;
  PLATINE_MOVE_POWER_MODE   movePowerMode;
} PLATINE_AXIS_POWER_PARAMS;

typedef struct  PLATINE_POWER_PARAMS {
  PLATINE_AXIS_POWER_PARAMS   pan;
  PLATINE_AXIS_POWER_PARAMS   tilt;
} PLATINE_POWER_PARAMS;

typedef struct PLATINE_OFFSETS_STR {
  double       pan;
  double       tilt;
} PLATINE_OFFSETS_STR;

/** ********************************************************************* **
 **                      ENSEMBLE DES PARAMETRES                          **
 ** ********************************************************************* **/

typedef struct PLATINE_PARAMS_STR {
  PLATINE_CONST_PARAMS    consts;
  PLATINE_OFFSETS_STR     offsets;
  PLATINE_SPEED_PROFILE   speedProfile;
  PLATINE_POWER_PARAMS    power;
} PLATINE_PARAMS_STR;


typedef enum PLATINE_SET_SETTING {
  PLATINE_SAVE_SETTING,            
  PLATINE_RESTORE_SAVED_SETTING,   
  PLATINE_RESTORE_FACTORY_SETTING
} PLATINE_SET_SETTING;

#endif /* PLATINE_STRUCT_H */
